%momentum
% GUI which enables automatic computation of AFTER state for a 2D two-body
% collision problem. Inputs are initial velocities u1,u2 (x,y vectors),
% masses m1, m2 and coefficient of restitution C.
%
% C is the speed of separation/speed of approach in direction of collision
% lines, which is the x (horizontal) axis.
%
% Outputs are velocities v1, v2 post collision, and the energy loss.
%
% If the masses are not actually approaching each other, no outputs will be
% returned. The info bar at the bottom of the GUI will warn of this
% situation. Otherwise it will give speed of separation / speed of
% approach.
%
% LAST UPDATED by Andy French Dec 2021

function varargout = momentum(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @momentum_OpeningFcn, ...
    'gui_OutputFcn',  @momentum_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% momentum opening function. This executes just before momentum is made visible.
function momentum_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for momentum
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;
d.legend = [];

%Center figure
figcentre( d.h.FIGUREmomentum );

%Set defaults
momentum_defaults

%Update momentum
momentum_update;

%%

% Default parameters
function momentum_defaults
global d

%Font size for graphs
d.fsize = 18;

%Arrow size for velocities
d.arrowsize = 2;
d.arrowheadsize = 0.2;

%Size of circles for masses and centre offsets
d.circle_radius = 1;
d.circle_offset = 3;

%Default parameter menu
d.default_menu = {
    'Elastic equal masses',...
    'Elastic big and small',...
    'Inelastic #1',...
    'C=0.5'
    };
set( d.h.MENUdefaults,'string',d.default_menu);

%Get menu option
d.menu_option = d.default_menu{ get(d.h.MENUdefaults,'value') };

%Two-body system defaults
if strcmp( d.menu_option,'Elastic equal masses' )
    d.u1x = 1; d.u1y = 0; d.u2x = -1; d.u2y = 0;
    d.m1 = 1; d.m2 = 1; d.C = 1;
elseif strcmp( d.menu_option,'Elastic big and small' )
    d.u1x = 2; d.u1y = 0; d.u2x = 0; d.u2y = 0;
    d.m1 = 5; d.m2 = 1; d.C = 1;
elseif strcmp( d.menu_option,'Inelastic #1' )
    d.u1x = 2; d.u1y = 0; d.u2x = 0; d.u2y = 0;
    d.m1 = 2; d.m2 = 3; d.C = 0;
elseif strcmp( d.menu_option,'C=0.5' )
    d.u1x = 2; d.u1y = 0; d.u2x = 0; d.u2y = 0;
    d.m1 = 1; d.m2 = 5; d.C = 0.5;
else
    %As per Elastic equal masses
    d.u1x = 1; d.u1y = 0; d.u2x = -1; d.u2y = 0;
    d.m1 = 1; d.m2 = 1; d.C = 1;
end

%Determine AFTER collision scenario
[d.v1x,d.v1y,d.v2x,d.v2y,d.dE,...
    d.collision,d.approach_speed,d.separation_speed,d.V] = collide( d.u1x,d.u1y,d.u2x,d.u2y,d.m1,d.m2,d.C );

%Set GUI with defaults
set( d.h.EDITu1x,'string',num2str(d.u1x) );
set( d.h.EDITu1y,'string',num2str(d.u1y) );
set( d.h.EDITu2x,'string',num2str(d.u2x) );
set( d.h.EDITu2y,'string',num2str(d.u2y) );
set( d.h.EDITv1x,'string',strrep( rats(d.v1x),' ','') );
set( d.h.EDITv1y,'string',num2str(d.v1y) );
set( d.h.EDITv2x,'string',strrep( rats(d.v2x),' ','') );
set( d.h.EDITv2y,'string',num2str(d.v2y) );
set( d.h.EDITm1,'string',num2str(d.m1) );
set( d.h.EDITm2,'string',num2str(d.m2) );
set( d.h.EDITC,'string',num2str(d.C) );
set( d.h.AXESbefore,'fontsize',d.fsize );
set( d.h.AXESafter,'fontsize',d.fsize );
set( d.h.TEXTenergyloss,'string',['Energy loss /J = ',num2str(d.dE)] );
if d.collision ==1
    set(d.h.TEXTinfo,'string', ['speed of separation / speed of approach = ',...
        num2str(d.separation_speed),'/',num2str(d.approach_speed),...
        '. ZMF velocity V = (',num2str(d.V(1)),',',num2str(d.V(2)),').']);
else
    set(d.h.TEXTinfo,'string', 'Objects are not approaching. No collision!');
end

%%

%2D collision problem calculator
%Assume collision line is the x direction. y velocities directions are unchanged.
function [v1x,v1y,v2x,v2y,dE,...
    collision,approach_speed,separation_speed,V] = collide( u1x,u1y,u2x,u2y,m1,m2,C )

%Form column vectors of velocities
u1 = [u1x;u1y];
u2 = [u2x;u2y];

%Determine whether objects are actually colliding
if dot( u2-u1,u1 ) >= 0
    collision = 0;
    %There is no collision!
    v1x = NaN;
    v1y = NaN;
    v2x = NaN;
    v2y = NaN;
    dE = NaN;
    approach_speed = NaN;
    separation_speed = NaN;
    return
else
    collision = 1;
end

%Determine AFTER collision velocities and energy loss
u1 = [u1x;u1y]; u2 = [u2x;u2y];
V = ( m1*u1 + m2*u2 )/(m1+m2);  %Zero Momentum Frame velocity
v1 = -C*( u1 - V ) + V; v1x = v1(1); v1y = v1(2);
v2 = -C*( u2 - V ) + V; v2x = v2(1); v2y = v2(2);
if C==1
    %Elastic collision
    dE = 0;
else
    dE = 0.5*m1*dot(u1,u1) + 0.5*m2*dot(u2,u2) - 0.5*m1*dot(v1,v1) - 0.5*m2*dot(v2,v2);
end
approach_speed = abs( u2x - u1x );
separation_speed = abs( v2x - v1x );

%%

%% Update GUI
function momentum_update
global d

%Get GUI parameters
d.u1x = str2num( get( d.h.EDITu1x,'string' ));
d.u1y = str2num( get( d.h.EDITu1y,'string' ));
d.u2x = str2num( get( d.h.EDITu2x,'string' ));
d.u2y = str2num( get( d.h.EDITu2y,'string' ));
d.m1 = str2num( get( d.h.EDITm1,'string' ));
d.m2 = str2num( get( d.h.EDITm2,'string' ));
d.C = str2num( get( d.h.EDITC,'string' ));

%Determine AFTER collision scenario
[d.v1x,d.v1y,d.v2x,d.v2y,d.dE,...
    d.collision,d.approach_speed,d.separation_speed,d.V] = collide( d.u1x,d.u1y,d.u2x,d.u2y,d.m1,d.m2,d.C );

%Update GUI
set( d.h.EDITv1x,'string',strrep( rats(d.v1x),' ','') );
set( d.h.EDITv1y,'string',num2str(d.v1y) );
set( d.h.EDITv2x,'string',strrep( rats(d.v2x),' ','') );
set( d.h.EDITv2y,'string',num2str(d.v2y) );
set( d.h.TEXTenergyloss,'string',['Energy loss /J = ',num2str(d.dE)] );
if d.collision ==1
    set(d.h.TEXTinfo,'string', ['speed of separation / speed of approach = ',...
        num2str(d.separation_speed),'/',num2str(d.approach_speed),...
        '. ZMF velocity V = (',num2str(d.V(1)),',',num2str(d.V(2)),').']);
else
    set(d.h.TEXTinfo,'string', 'Objects are not approaching. No collision!');
end

%Clear axes and plot collision diagrams

%BEFORE
axes(d.h.AXESbefore);
cla;
drawarrow( -d.circle_offset, -d.circle_offset + d.u1x  , 0, d.u1y, d.arrowheadsize, [1,0,0],d.arrowsize,d.circle_radius );
drawarrow( d.circle_offset, d.circle_offset+d.u2x  , 0, d.u2y, d.arrowheadsize, [0,1,0],d.arrowsize,d.circle_radius );
axis equal
axis tight
axis off
xlimits_before = get(d.h.AXESbefore,'xlim');
ylimits_before = get(d.h.AXESbefore,'ylim');

%AFTER
axes(d.h.AXESafter);
cla;
drawarrow( -d.circle_offset, -d.circle_offset + d.v1x  , 0, d.v1y, d.arrowheadsize, [1,0,0],d.arrowsize,d.circle_radius );
drawarrow( d.circle_offset, d.circle_offset + d.v2x  , 0, d.v2y, d.arrowheadsize, [0,1,0],d.arrowsize,d.circle_radius );
axis equal
axis tight
axis off
xlimits_after = get(d.h.AXESafter,'xlim');
ylimits_after = get(d.h.AXESafter,'ylim');

%Set axis scale to be the same, so circles look the same size
set( d.h.AXESafter,'xlim',[min([xlimits_before,xlimits_after]),...
    max([xlimits_before,xlimits_after])] );
set( d.h.AXESafter,'ylim',[min([ylimits_before,ylimits_after]),...
    max([ylimits_before,ylimits_after])] );
set( d.h.AXESbefore,'xlim',[min([xlimits_before,xlimits_after]),...
    max([xlimits_before,xlimits_after])] );
set( d.h.AXESbefore,'ylim',[min([ylimits_before,ylimits_after]),...
    max([ylimits_before,ylimits_after])] );

%%

%% Output function. Outputs from this function are returned to the command line.
function varargout = momentum_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Hide AFTER figure
function CHECKhideAFTER_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhideAFTER,'value') == 1
    set( d.h.TEXTbiggraybox,'visible','on' );
else
    set( d.h.TEXTbiggraybox,'visible','off' );
end

%%

%% Hide u1 checkbox toggle
function CHECKhideu1_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhideu1,'value') == 1
    set( d.h.EDITu1x,'visible','off' );
    set( d.h.EDITu1y,'visible','off' );
else
    set( d.h.EDITu1x,'visible','on' );
    set( d.h.EDITu1y,'visible','on' );
end

%%

%% Hide u2 checkbox toggle
function CHECKhideu2_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhideu2,'value') == 1
    set( d.h.EDITu2x,'visible','off' );
    set( d.h.EDITu2y,'visible','off' );
else
    set( d.h.EDITu2x,'visible','on' );
    set( d.h.EDITu2y,'visible','on' );
end

%%

%% Hide v1 checkbox toggle
function CHECKhidev1_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhidev1,'value') == 1
    set( d.h.EDITv1x,'visible','off' );
    set( d.h.EDITv1y,'visible','off' );
else
    set( d.h.EDITv1x,'visible','on' );
    set( d.h.EDITv1y,'visible','on' );
end

%%

%% Hide v2 checkbox toggle
function CHECKhidev2_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhidev2,'value') == 1
    set( d.h.EDITv2x,'visible','off' );
    set( d.h.EDITv2y,'visible','off' );
else
    set( d.h.EDITv2x,'visible','on' );
    set( d.h.EDITv2y,'visible','on' );
end

%%

%% Hide m1 checkbox toggle
function CHECKhidem1_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhidem1,'value') == 1
    set( d.h.EDITm1,'visible','off' );
else
    set( d.h.EDITm1,'visible','on' );
end

%%

%% Hide m2 checkbox toggle
function CHECKhidem2_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhidem2,'value') == 1
    set( d.h.EDITm2,'visible','off' );
else
    set( d.h.EDITm2,'visible','on' );
end

%%

%% Hide energy change textbox toggle
function CHECKhidedE_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhidedE,'value') == 1
    set( d.h.TEXTenergyloss,'visible','off' );
else
    set( d.h.TEXTenergyloss,'visible','on' );
end

%%

%% Hide info box textbox toggle
function CHECKhideinfo_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhideinfo,'value') == 1
    set( d.h.TEXTinfo,'visible','off' );
else
    set( d.h.TEXTinfo,'visible','on' );
end

%%

%% Hide coefficient of restitution editbox
function CHECKhidecor_Callback(hObject, eventdata, handles)
global d
if get(d.h.CHECKhidecor,'value') == 1
    set( d.h.EDITC,'visible','off' );
else
    set( d.h.EDITC,'visible','on' );
end

%%

%% u1x component edit box
function EDITu1x_Callback(hObject, eventdata, handles)
global d
d.u1x = str2num( get( d.h.EDITu1x,'string' ));
momentum_update;

%%

%% u1y component edit box
function EDITu1y_Callback(hObject, eventdata, handles)
global d
d.u1y = str2num( get( d.h.EDITu1y,'string' ));
momentum_update;

%%

%% u2x component edit box
function EDITu2x_Callback(hObject, eventdata, handles)
global d
d.u2x = str2num( get( d.h.EDITu2x,'string' ));
momentum_update;

%%

%% u2y component edit box
function EDITu2y_Callback(hObject, eventdata, handles)
global d
d.u2y = str2num( get( d.h.EDITu2y,'string' ));
momentum_update;

%%

%% m1 edit box
function EDITm1_Callback(hObject, eventdata, handles)
global d
d.m1 = str2num( get( d.h.EDITm1,'string' ));
momentum_update;

%%

%% m2 edit box
function EDITm2_Callback(hObject, eventdata, handles)
global d
d.m2 = str2num( get( d.h.EDITm2,'string' ));
momentum_update;

%%

%% Coefficient of restitution edit box
function EDITC_Callback(hObject, eventdata, handles)
global d
d.C = str2num( get( d.h.EDITC,'string' ));
momentum_update;

%%

%Defaults menu
function MENUdefaults_Callback(hObject, eventdata, handles)
momentum_defaults;
momentum_update;

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%%

%drawarrow
% Function which draws an arrow defined by start
% and end coordinates [x0,x1] and [y0,y1]. Draws the arrow emmanating
% from a filled circle of radius r
function a = drawarrow( x0, x1, y0, y1, arrowsize, arrow_RGB, linesize, circle_radius )

%Rotation angle of arrow from horizontal in order for arrow to point along
%line
t = atan2( y1-y0,x1-x0 );

%Define centre of arrowhead
xc = x1 + circle_radius*cos(t);
yc = y1 + circle_radius*sin(t);

%Transform arrow coordinates
a = arrowsize;
r = [ 0, 2*a, a; 0, 0, 2*a];
r = r - [a,a,a;a,a,a];
r = [ sin(t), cos(t); -cos(t), sin(t)] * r ;
r = r + [xc,xc,xc; yc,yc,yc];

%Define and plot circle
phi = linspace(0,2*pi,500);
x = circle_radius*cos(phi);
y = circle_radius*sin(phi);
a(1) = patch( x+x0,y+y0,arrow_RGB ); hold on;
set(a(1),'Edgecolor',arrow_RGB );

%Plot line
a(2) = plot( [x0+ circle_radius*cos(t),x1+ circle_radius*cos(t)],...
    [y0+ circle_radius*sin(t),y1+ circle_radius*sin(t)],'color',arrow_RGB,'linewidth',linesize );

%Plot arrowhead
if (x0==x1)&&(y0==y1)
    %Arrow length is zero, so don't plot arrowhead
    a(3) = 0;
else
    a(3) = patch(r(1,:),r(2,:),arrow_RGB);
    set(a(3),'Edgecolor',arrow_RGB );
end

%End of code






