% movie2xyt
% GUI which enables a video file to be loaded and stepped through
% frame-by-frame using a slider or arrow keys. By positioning a
% semi-transparent red blob over an object of interest, an x,y,t trajectory
% can be determined. The programe will then use cublic-spline interpolation
% to determine x and y velocity and acceleration vs time curves for the
% object. Graphs are plotted and an Excel sheet of the kinematic data will
% be outputted.
%
% If you are running a PC it can load AVI (.avi), MPEG-1 (.mpg),
% Windows Media Video (.wmv, .asf, .asx), and any format supported
% by Microsoft DirectShow.
%
% If you are running a Macintosh system it can load AVI (.avi), MPEG-1 (.mpg),
% MPEG-4 (.mp4, .m4v), Apple QuickTime Movie (.mov).
%
% If your video needs conversion, try http://video.online-convert.com/
%
% LAST UPDATED by Andy French September 2016.

%%%% GUI INITIALIZATION %%%%

%Top level GUI initialization
function varargout = movie2xyt(varargin)

% Generic MATLAB GUI initialization. Edit at your peril!
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @movie2xyt_OpeningFcn, ...
    'gui_OutputFcn',  @movie2xyt_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

% Outputs from this function are returned to the command line.
function varargout = movie2xyt_OutputFcn(hObject, eventdata, handles)
varargout{1} = handles.output;

%%

% Code which executes just before movie2xyt is made visible.
function movie2xyt_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for movie2xyt
handles.output = hObject;
guidata(hObject, handles);

%Add all subdirectories within current directory to MATLAB path
addpath( genpath(pwd), '-begin');

%Store GUI object handles data in a global array d which can be seen (and
%modified) by all functions. This will be the main GUI data array which
%enables all objects to communicate with one another.
global d
d.h = handles; clear handles

%Centre figure
figcentre(d.h.FIGUREmovie2xyt);

%Add key-press handler to figure and close-request function. The code for
%these operations is to be found later in this file. Also add mouse button event
%controls (click down, drag, click up etc) to figure.
set( d.h.FIGUREmovie2xyt,...
    'KeyPressFcn',@keypressfunc,...
    'KeyReleaseFcn',@keyreleasefunc,...
    'CloseRequestFcn',@closerequestfunc,...
    'WindowButtonDownFcn',@mouseclickdown,...
    'WindowButtonMotionFcn',@mousedrag,...
    'WindowButtonUpFcn',@mouseclickup,...
    'WindowScrollWheelFcn',@mousewheelscroll );

%Set figure renderer
set( d.h.FIGUREmovie2xyt,'renderer','opengl');

%Set default name for GUI
d.name = 'movie2xyt';
set( d.h.FIGUREmovie2xyt,'name',d.name);

%Initialize AXES and set axes units to be pixels and set the axes
%outerposition to be the same as the plotting region.
axes(d.h.AXES); axis off;
set( d.h.AXES,'Nextplot','add','units','pixels');
axis equal

%Initialize mouse parameters
d.mouse.x = [];
d.mouse.y = [];
d.mouse.within_axes = 0;
d.mouse.button = 'not clicked';

%Initialize movie parameters
d.movie.filename = '';
d.movie.movie = [];
d.movie.frame = [];
d.movie.deafult_frame_rate = 30;

%Initialize parameters of red blob object tracker
d.redblob.colour = [1,0,0];
d.redblob.alpha = 0.5;
d.redblob.handle = [];
d.show_blob = 0;

%Initialise x,y coordinates of points which define yellow distance scale
d.scale.x0 = 0;
d.scale.x1 = 1;
d.scale.y0 = 0;
d.scale.y1 = 0;
d.scale.handle = [];
d.scale.length_m = 1;
set( d.h.EDITeditcalibration,'string',num2str(d.scale.length_m) );

%Default calibration or move red blob tracker mode
d.calibration_mode = 1;
d.show_calibration = 0;
set( d.h.TOGGLEeditcalibration, 'value', d.calibration_mode );

%Initialize output kinematic data arrays. i means interpolated.
d.kinematics.t = [];
d.kinematics.x = [];
d.kinematics.y = [];
d.kinematics.vx = [];
d.kinematics.vy = [];
d.kinematics.ax = [];
d.kinematics.ay = [];
d.kinematics.ti = [];
d.kinematics.xi = [];
d.kinematics.yi = [];
d.kinematics.vxi = [];
d.kinematics.vyi = [];
d.kinematics.axi = [];
d.kinematics.ayi = [];

%Set number of interpolated outputs for kinematic data
d.kinematics.N = 1000;

%%

%%% GUI CALLBACK FUNCTIONS %%%%

%%% MOVIE %%%

% Load movie file (e.g .mp4, .avi etc) pushbutton
function PUSHloadmovie_Callback(hObject, eventdata, handles)
global d

%Bring up a windows file browser
if ispc==1
[filename, pathname] = uigetfile( {'*.mp4;*.mj2;*.avi;*.mgp;*.wmv;*.asf;*.asx'},...
    'Choose movie file to load');
elseif ismac==1
    [filename, pathname] = uigetfile( {'*.mj2;*.avi;*.mgp;*.wmv;*.asf;*.asx;*.mp4;*.mov;*.ogg'},...
    'Choose movie file to load');
    
elseif isunix==1
    [filename, pathname] = uigetfile( {'*.mp4;*.mj2;*.avi;*.mgp;*.wmv;*.asf;*.asx;*.mp4;*.mov;*.ogg'},...
    'Choose movie file to load');
else
    return
end

%Only proceed if file selected ...
if filename ~=0
    
    %Store the filename in the GUI data
    d.movie.filename = [pathname,filename];
    
    %Update the edit box with the filename only
    set( d.h.EDITloadmovie, 'string', filename );
    
    %Load the movie
    readerobj = VideoReader( d.movie.filename, 'tag', 'movie2xyt');
    
    % Read in all video frames.
    vidFrames = read(readerobj);
    
    % Get the number of frames and frame rate
    d.movie.numFrames = get(readerobj, 'NumberOfFrames');
    d.movie.FrameRate = readerobj.FrameRate;
    if d.movie.FrameRate == 0
        d.movie.FrameRate = d.movie.deafult_frame_rate;
    end
    set( d.h.TEXTframerate,'string',[ 'Frame rate = ',num2str(d.movie.FrameRate),' FPS' ] );
    
    %Get the frame size in pixels
    d.movie.Width = readerobj.Width;
    d.movie.Height = readerobj.Height;
    
    % Create a MATLAB movie struct from the video frames.
    for k = 1 : d.movie.numFrames
        d.movie.movieframes(k).cdata = vidFrames(:,:,:,k);
        d.movie.movieframes(k).colormap = [];
    end
    
    %Set movie frame number to be 1
    d.movie.frame = 1;
    set( d.h.TEXTframe,'string',['Frame ',num2str(d.movie.frame),' of ',num2str(d.movie.numFrames)] );
    
    %Clear unwanted variabls
    clear readerobj vidFrames k
    
    %Display first frame in AXES
    axes( d.h.AXES );
    d.image = image( flipdim( d.movie.movieframes(1).cdata, 1 ) );
    d.image_width = get( d.image,'xdata');
    d.image_width = d.image_width(2);
    d.image_height = get( d.image,'ydata');
    d.image_height = d.image_height(2);
    drawnow
    axis manual
    
    %Create red blob tracker. Make this about 5% of the screen size.
    hold on;
    d.redblob.xc = zeros(1,d.movie.numFrames);
    d.redblob.yc = zeros(1,d.movie.numFrames);
    d.redblob.r = 0.05*min( [d.image_width,d.image_height] );
    d.redblob.t = linspace( 0, 2*pi, 100 );
    d.redblob.x = d.redblob.r*cos(d.redblob.t);
    d.redblob.y = d.redblob.r*sin(d.redblob.t);
    d.redblob.handle = patch( d.redblob.x, d.redblob.y, d.redblob.colour );
    set( d.redblob.handle,'facealpha',d.redblob.alpha,'edgecolor', d.redblob.colour );
    
    %Create calibration scale. Initialize this at half the screen width.
    hold on;
    d.scale.x0 = d.image_width/4;
    d.scale.x1 = 3*d.image_width/4;
    d.scale.y0 = d.image_height/6;
    d.scale.y1 = d.image_height/6;
    d.scale.handle = plot( [d.scale.x0,d.scale.x1], [d.scale.y0,d.scale.y1], 'y' );
    set( d.scale.handle,'linewidth',2 );
    d.scale.pixels = d.image_width/2;
    
    %Update kinematics data arrays, there will be one data point per video frame
    d.kinematics.t = 0 : 1/d.movie.FrameRate : (d.movie.numFrames - 1)/d.movie.FrameRate;
    d.kinematics.x = zeros(1,d.movie.numFrames);
    d.kinematics.y = zeros(1,d.movie.numFrames);
    d.kinematics = update_kinematics( d.kinematics.x, d.kinematics.y,...
        d.kinematics.t, d.kinematics.N );
    
    %Initialize time /s
    set( d.h.TEXTtime,'string',[num2str( d.kinematics.t(d.movie.frame) ),'s'] );
end

%%

% Open loaded movie file externally to MATLAB using the default Windows
% program for the movie file file extension
function PUSHmovieopen_Callback(hObject, eventdata, handles)
global d

%Open file using default windows program
if strcmp( d.movie.filename,'') ~=1
    winopen( d.movie.filename );
end

%%

%%%% KEYBOARD & MOUSE INTERACTION CALLBACKS %%%%

% e.g. key press handler, mouse click event manager,
% close request function etc

%Key press handler
function keypressfunc(hObject, eventdata, handles)
global d

%Get the string of the current key press
s = get( d.h.FIGUREmovie2xyt,'currentkey');

%Actions, depending on the key pressed...

if ( strcmp(s,'q') == 1 ) || ( strcmp(s,'Q') == 1 )
    %Quit GUI
    closerequestfunc;
    
elseif ( strcmp(s,'rightarrow') == 1 ) || ( strcmp(s,'downarrow') == 1 )
    %Advance frame
    if isempty( d.movie.frame )==0
        d.movie.frame = d.movie.frame + 1;
        if d.movie.frame > d.movie.numFrames
            d.movie.frame = d.movie.numFrames;
        end
        set( d.image, 'cdata', flipdim( d.movie.movieframes(d.movie.frame).cdata, 1 ) );
        set( d.h.TEXTframe,'string',['Frame ',num2str(d.movie.frame),' of ',num2str(d.movie.numFrames)] );
        set( d.h.TEXTtime,'string',[num2str( d.kinematics.t(d.movie.frame) ),'s'] );
        d.redblob.x = d.redblob.xc(d.movie.frame) +  d.redblob.r*cos(d.redblob.t);
        d.redblob.y = d.redblob.yc(d.movie.frame) +  d.redblob.r*sin(d.redblob.t);
        set( d.redblob.handle, 'xdata',d.redblob.x, 'ydata',d.redblob.y );
    end
    
elseif ( strcmp(s,'leftarrow') == 1 ) || ( strcmp(s,'uparrow') == 1 )
    %Previous frame
    if isempty( d.movie.frame )==0
        d.movie.frame = d.movie.frame - 1;
        if d.movie.frame < 1
            d.movie.frame = 1;
        end
        set( d.image, 'cdata', flipdim( d.movie.movieframes(d.movie.frame).cdata, 1 ) );
        set( d.h.TEXTframe,'string',['Frame ',num2str(d.movie.frame),' of ',num2str(d.movie.numFrames)] );
        set( d.h.TEXTtime,'string',[num2str( d.kinematics.t(d.movie.frame) ),'s'] );
        d.redblob.x = d.redblob.xc(d.movie.frame) +  d.redblob.r*cos(d.redblob.t);
        d.redblob.y = d.redblob.yc(d.movie.frame) +  d.redblob.r*sin(d.redblob.t);
        set( d.redblob.handle, 'xdata',d.redblob.x, 'ydata',d.redblob.y );
    end
end

%%

%Key release handler
function keyreleasefunc(hObject, eventdata, handles)
global d

%%

%This function executes when the mouse is clicked
function mouseclickdown(hObject, eventdata, handles)
global d

%Get click-type
d.mouse.clicktype = get(d.h.FIGUREmovie2xyt, 'SelectionType');

%Double left mouse button click
if strcmp(d.mouse.clicktype,'open')==1
    d.mouse.button = 'double click';
    
    %Single left mouse click
elseif strcmp(d.mouse.clicktype,'normal')==1
    d.mouse.button = 'single click';
    
    %Shift + click or right & left mouse click
elseif strcmp(d.mouse.clicktype,'extend')==1
    d.mouse.button = 'shift & click';
    
    %Control + click or right mouse click
elseif strcmp(d.mouse.clicktype,'alt')==1
    d.mouse.button = 'ctrl & click';
else
    d.mouse.button = 'not clicked';
end

%Store 'click point' coordinates (pixels from bottom left of the figure)
cp=get( d.h.FIGUREmovie2xyt,'currentpoint');
d.mouse.x = cp(1,1);
d.mouse.y = cp(1,2);

%Check 'click point' coordinates are within axes
ax_pos = get( d.h.AXES, 'position' );
xv = [ ax_pos(1), ax_pos(1)+ax_pos(3), ax_pos(1)+ax_pos(3), ax_pos(1) ];
yv = [ ax_pos(2), ax_pos(2), ax_pos(2) + ax_pos(4), ax_pos(2) + ax_pos(4) ];
d.mouse.within_axes = inpolygon( d.mouse.x, d.mouse.y, xv, yv );

%Obtain current point in axes coordinates
cp=get( d.h.AXES,'currentpoint');
d.mouse.x = cp(1,1);
d.mouse.y = cp(1,2);

%Has mouse been clicked within 10 pixels of centre of red blob?
delta = sqrt( ( d.mouse.x - d.redblob.xc(d.movie.frame) )^2 + ( d.mouse.y - d.redblob.yc(d.movie.frame) )^2 );
if delta <=10
    d.redblob.selected = 1;
    d.redblob.xc0 = d.redblob.xc(d.movie.frame);
    d.redblob.yc0 = d.redblob.yc(d.movie.frame);
else
    d.redblob.selected = 0;
end

%Has the mouse been clicked within 10 pixels of the start or end of the
%calibration scale?
delta = sqrt( (d.mouse.x - d.scale.x0)^2 + (d.mouse.y - d.scale.y0)^2 );
if delta <=10
    d.scale.start_selected = 1;
else
    d.scale.start_selected = 0;
end
delta = sqrt( (d.mouse.x - d.scale.x1)^2 + (d.mouse.y - d.scale.y1)^2 );
if delta <=10
    d.scale.end_selected = 1;
else
    d.scale.end_selected = 0;
end

%%

%This function runs when a mouse is dragged following a click event, and
%before the mouse button is released
function mousedrag(hObject, eventdata, handles)
global d

%Store 'click point' coordinates (pixels from bottom left of the figure)
cp=get( d.h.AXES,'currentpoint');
x = cp(1,1);
y = cp(1,2);

%Determine pixel displacement from click point
dx = x - d.mouse.x;
dy = y - d.mouse.y;

%Move tracker blob if mouse was originally clicked within AXES and
%calibration mode not activated
if ( d.mouse.within_axes==1 ) && ( d.calibration_mode == 0 ) && ( isempty( d.movie.frame) == 0 ) && ( d.redblob.selected == 1 )
    set( d.redblob.handle, 'Xdata', d.redblob.x + dx, 'Ydata', d.redblob.y + dy );
    d.redblob.xc(d.movie.frame) = d.redblob.xc0 + dx ;
    d.redblob.yc(d.movie.frame) = d.redblob.yc0 + dy ;
    drawnow;
end

%Move start of calibration scale if calibration mode is on and mouse click was
%within 100 pixels of the start of the calibration scale
if ( d.mouse.within_axes==1 ) && ( d.calibration_mode == 1 ) && ( isempty( d.movie.frame) == 0 ) && ( d.scale.start_selected == 1 )
    set( d.scale.handle, 'xdata',[ d.scale.x0 + dx, d.scale.x1 ], 'ydata',[ d.scale.y0 + dy, d.scale.y1 ] );
end

%Move end of calibration scale if calibration mode is on and mouse click was
%within 100 pixels of the start of the calibration scale
if ( d.mouse.within_axes==1 ) && ( d.calibration_mode == 1 ) && ( isempty( d.movie.frame) == 0 ) && ( d.scale.end_selected == 1 )
    set( d.scale.handle, 'xdata',[ d.scale.x0, d.scale.x1 + dx ], 'ydata',[ d.scale.y0, d.scale.y1 + dy] );
end

%%

%This function executes when a mouse button is released
function mouseclickup(hObject, eventdata, handles)
global d

%Reset mouse parameters
d.mouse.within_axes = 0;
d.mouse.button = 'not clicked';

%Update calibration scale coordinates
if ( isempty( d.movie.frame) == 0 )
    if ( d.calibration_mode == 1 )
        %Update calibration scale coordinates
        xdata = get( d.scale.handle, 'xdata' );
        ydata = get( d.scale.handle, 'ydata' );
        d.scale.x0 = xdata(1);
        d.scale.x1 = xdata(2);
        d.scale.y0 = ydata(1);
        d.scale.y1 = ydata(2);
        d.scale.pixels = sqrt( (d.scale.x1 - d.scale.x0)^2 + (d.scale.y1 - d.scale.y0)^2 );
    end
end

%%

%This function executes when the mouse scroll wheel is used
function mousewheelscroll(hObject, eventdata, handles)
global d

%This doesn't do anything yet - but could be used to advance or go to
%previous frames

%%

%Edit calibration toggle button
function TOGGLEeditcalibration_Callback(hObject, eventdata, handles)
global d
d.calibration_mode = get( d.h.TOGGLEeditcalibration, 'value' );

%Depending on the mode, change the string
if d.calibration_mode==1
    set(d.h.TOGGLEeditcalibration,'Backgroundcolor',[1,1,0],'string','Edit calibration');
else
    set(d.h.TOGGLEeditcalibration,'Backgroundcolor',[1,0,0],'string','Edit tracker');
end

%%

%Save tracker data pushbutton
function PUSHsavetracker_Callback(hObject, eventdata, handles)
global d

%Construct default filename
filename = ['movie2xyt data ',strrep( datestr(now),':','-' )];
[filename,pathname] = uiputfile( '*.mat', 'Save tracker data file', filename );

%Export .mat file
if ( filename == 0 )
    return
elseif ( isempty(d.movie.frame)==0 )
    f.scale = d.scale;
    f.redblob = d.redblob;
    f.kinematics = d.kinematics;
    save( filename, '-struct', 'f' );
    clear f
end

%%

%Load tracker data pushbutton
function PUSHloadtracker_Callback(hObject, eventdata, handles)
global d

%Bring up a windows file browser
[filename, pathname] = uigetfile( '*.mat','Choose tracker data file');

%Only proceed if file selected ...
if (filename ==0 )
    return
elseif ( isempty(d.movie.frame)==1 )
    return
else
    
    %Load tracker file
    f = load( [pathname,filename] );
    
    %Check that tracker file matches the number of frames in the current movie
    if ( d.movie.numFrames ~= numel( f.redblob.xc ) )
        errordlg(' Tracker file chosen does not have the same number of frames as the current movie')
        return
    end
    
    %Update kinematics
    d.kinematics = f.kinematics;
    
    %Update calibration scale
    d.scale.x0 = f.scale.x0;
    d.scale.x1 = f.scale.x1;
    d.scale.y0 = f.scale.y0;
    d.scale.y1 = f.scale.y1;
    d.scale.length_m = f.scale.length_m;
    d.scale.pixels = sqrt( (d.scale.x1 - d.scale.x0)^2 + (d.scale.y1 - d.scale.y0)^2 );
    set( d.h.EDITeditcalibration,'string',num2str(d.scale.length_m) );
    set( d.scale.handle, 'xdata',[ d.scale.x0, d.scale.x1], 'ydata',[ d.scale.y0, d.scale.y1] );
    
    %Update red blob tracker
    d.redblob.xc = f.redblob.xc;
    d.redblob.yc = f.redblob.yc;
    d.redblob.x = d.redblob.xc(d.movie.frame) +  d.redblob.r*cos(d.redblob.t);
    d.redblob.y = d.redblob.yc(d.movie.frame) +  d.redblob.r*sin(d.redblob.t);
    set( d.redblob.handle, 'xdata',d.redblob.x, 'ydata',d.redblob.y );
end

%%

%Save kinematic data
function PUSHsavexyt_Callback(hObject, eventdata, handles)
global d

%Set fontsize for plots
fsize = 16;

%Construct default filename
filename = ['kinematics ',strrep( datestr(now),':','-' )];
[filename,pathname] = uiputfile( '*.xls', 'Save kinematics data to .xls',filename );

%Check if file specified, or indeed a movie has been loaded
if ( filename == 0 )
    return
end
if ( isempty(d.movie.frame)==1 )
    return
end

%Update kinematic parameters
d.kinematics.x = d.redblob.xc*d.scale.length_m/d.scale.pixels;
d.kinematics.y = d.redblob.yc*d.scale.length_m/d.scale.pixels;
d.kinematics = update_kinematics( d.kinematics.x, d.kinematics.y, d.kinematics.t, d.kinematics.N );

%Reassign kinematics data for brevity
k = d.kinematics;

%Construct cell array(s) to be written to Excel
E = { ['Kinematics data for file: ',filename] };
E{3,1} = 'Frame';
E{3,2} = 't /s';
E{3,3} = 'x /m';
E{3,4} = 'y /m';
E{3,5} = 'vx /ms^-1';
E{3,6} = 'vy /ms^-1';
E{3,7} = 'ax /ms^-2';
E{3,8} = 'ay /ms^-2';
for n=1:d.movie.numFrames
    E{3+n,1} = n;
    E{3+n,2} = k.t(n);
    E{3+n,3} = k.x(n);
    E{3+n,4} = k.y(n);
    E{3+n,5} = k.vx(n);
    E{3+n,6} = k.vy(n);
    E{3+n,7} = k.ax(n);
    E{3+n,8} = k.ay(n);
end
Ei = { ['Interpolated kinematics data for file: ',filename] };
Ei{3,1} = 't /s';
Ei{3,2} = 'x /m';
Ei{3,3} = 'y /m';
Ei{3,4} = 'vx /ms^-1';
Ei{3,5} = 'vy /ms^-1';
Ei{3,6} = 'ax /ms^-2';
Ei{3,7} = 'ay /ms^-2';
for n=1:k.N
    Ei{3+n,1} = k.ti(n);
    Ei{3+n,2} = k.xi(n);
    Ei{3+n,3} = k.yi(n);
    Ei{3+n,4} = k.vxi(n);
    Ei{3+n,5} = k.vyi(n);
    Ei{3+n,6} = k.axi(n);
    Ei{3+n,7} = k.ayi(n);
end

%Save Excel file
warning off
xlswrite( [pathname,filename], E, 'Frame data' );
xlswrite( [pathname,filename], Ei, 'Interpolated' );

%Remove .xls extension in filename
filename = strrep( filename,'.xls','');

%Contruct x,y vs t plots
figure;
plot( k.ti,k.xi,'r' ); hold on;
plot( k.t,k.x,'r+' );
plot( k.ti,k.yi,'b' );
plot( k.t,k.y,'b+' );
title({filename,'x and y vs t'},'fontsize',fsize);
xlabel('t /s','fontsize',fsize);
ylabel('displacement /metres','fontsize',fsize);
set(gca,'fontsize',fsize);
legend({'x','xi','y','yi'},'fontsize',fsize)
grid on;
print( gcf, [pathname,filename,' x vs y vs t.png'],'-dpng','-r300' );
clf;

%Construct vx,vy vs t plots
plot( k.ti,k.vxi,'r' ); hold on;
plot( k.t,k.vx,'r+' );
plot( k.ti,k.vyi,'b' );
plot( k.t,k.vy,'b+' );
title({filename,'vx and vy vs t'},'fontsize',fsize);
xlabel('t /s','fontsize',fsize);
ylabel('velocity /ms^{-1}','fontsize',fsize);
set(gca,'fontsize',fsize);
legend({'vx','vxi','vy','vyi'},'fontsize',fsize)
grid on;
print( gcf, [pathname,filename,' vx vs vy.png'],'-dpng','-r300' );
clf;

%Contruct ax,ay vs t plots
plot( k.ti,k.axi,'r' ); hold on;
plot( k.t,k.ax,'r+' );
plot( k.ti,k.ayi,'b' );
plot( k.t,k.ay,'b+' );
title({filename,'ax and ay vs t'},'fontsize',fsize);
xlabel('t /s','fontsize',fsize);
ylabel('acceleration /ms^{-2}','fontsize',fsize);
set(gca,'fontsize',fsize);
legend({'ax','axi','ay','ayi'},'fontsize',fsize)
grid on;
print( gcf, [pathname,filename,' ax vs ay.png'],'-dpng','-r300' );
clf;

%Contruct x vs yplots
plot( k.xi,k.yi,'r' ); hold on;
plot( k.x,k.y,'r+' );
plot( k.x(1),k.y(1),'g*' );
plot( k.x(end),k.y(end),'m*' );
axis equal
title({filename,'x vs y'},'fontsize',fsize);
xlabel('x /metres','fontsize',fsize);
ylabel('y /metres','fontsize',fsize);
set(gca,'fontsize',fsize);
grid on;
print( gcf, [pathname,filename,' x vs y.png'],'-dpng','-r300' );
close(gcf);

%%

%Define calibration length edit box
function EDITeditcalibration_Callback(hObject, eventdata, handles)
global d
d.scale.length_m = str2num( get( d.h.EDITeditcalibration, 'string' ) );

%%

%Advance frame
function PUSHadvanceframe_Callback(hObject, eventdata, handles)
global d
if isempty( d.movie.frame )==0
    d.movie.frame = d.movie.frame + 1;
    if d.movie.frame > d.movie.numFrames
        d.movie.frame = d.movie.numFrames;
    end
    set( d.image, 'cdata', flipdim( d.movie.movieframes(d.movie.frame).cdata, 1 ) );
    set( d.h.TEXTframe,'string',['Frame ',num2str(d.movie.frame),' of ',num2str(d.movie.numFrames)] );
    set( d.h.TEXTtime,'string',[num2str( d.kinematics.t(d.movie.frame) ),'s'] );
    d.redblob.x = d.redblob.xc(d.movie.frame) +  d.redblob.r*cos(d.redblob.t);
    d.redblob.y = d.redblob.yc(d.movie.frame) +  d.redblob.r*sin(d.redblob.t);
    set( d.redblob.handle, 'xdata',d.redblob.x, 'ydata',d.redblob.y );
end

%%

%Previous frame
function PUSHpreviousframe_Callback(hObject, eventdata, handles)
global d
if isempty( d.movie.frame )==0
    d.movie.frame = d.movie.frame - 1;
    if d.movie.frame < 1
        d.movie.frame = 1;
    end
    set( d.image, 'cdata', flipdim( d.movie.movieframes(d.movie.frame).cdata, 1 ) );
    set( d.h.TEXTframe,'string',['Frame ',num2str(d.movie.frame),' of ',num2str(d.movie.numFrames)] );
    set( d.h.TEXTtime,'string',[num2str( d.kinematics.t(d.movie.frame) ),'s'] );
    d.redblob.x = d.redblob.xc(d.movie.frame) +  d.redblob.r*cos(d.redblob.t);
    d.redblob.y = d.redblob.yc(d.movie.frame) +  d.redblob.r*sin(d.redblob.t);
    set( d.redblob.handle, 'xdata',d.redblob.x, 'ydata',d.redblob.y );
end

%%

%GUI close function
function closerequestfunc(hObject, eventdata, handles)
global d

%Close GUI
delete( d.h.FIGUREmovie2xyt );

%

%%%% OTHER FUNCTIONS %%%

%Update kinematics, based upon vectors of x, y and time (t) coordinates
%corresponding to the centre of the red blob for each frame. N is the
%number of data points for an interpolated set of kinematic parameters.
function k = update_kinematics( x, y, t, N )

%Store original data in structure k
k.t = t;
k.x = x;
k.y = y;
k.N = N;

%Interpolate x,y,t
k.ti = linspace( t(1),t(end),N);
k.xi = interp1( t,x, k.ti, 'spline' );
k.yi = interp1( t,y, k.ti, 'spline' );

%Differentiate x,y data to form velocities and accelerations
k.vx = differentiator( t, x, t );
k.vy = differentiator( t, y, t );
k.ax = differentiator( t, k.vx, t );
k.ay = differentiator( t, k.vy, t );

%Interpolate velocities and accelerations to enable a smooth plot to be
%formed
k.vxi = interp1( t,k.vx, k.ti, 'spline' );
k.vyi = interp1( t,k.vy, k.ti, 'spline' );
k.axi = interp1( t,k.ax, k.ti, 'spline' );
k.ayi = interp1( t,k.ay, k.ti, 'spline' );

%%

%differentiator
% Differentiation function using cubic splines.
%
% Syntax: dy_by_dx = differentiate( xx, yy, x )
%
% xx, yy     Samples from function y(x) which is being differentiated. These
%            must be the same dimensions and have at least three elements each.
% x          Point on y(x) at which the gradient is returned. This can be an array of any
%            dimensions.
%
% dy_by_dx   Output of gradient of y(x) corresponindg to array x
%
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.

function dy_by_dx = differentiator( xx, yy, x )

%Check inputs are the right size
if ( numel(xx) == numel(yy) ) && ( numel(xx) >= 3 )
    
    %Sort xx in ascending order
    [xx,i] = sort(xx);
    yy = yy(i);
    
    %Compute piecewise polynomial based upon cubic splines.
    pp=spline(xx,yy);
    %pp = pchip(xx,yy);  %Alternative Hermite polynomial fit. Apparently this
    %is slightly better at preserving the shape of the function near
    %discontinuities.
    
    %Take apart piecewise polynomial
    [br,co,npy,nco]=unmkpp(pp);
    
    %Scale factors for differentiation
    sf=nco-1:-1:1;
    
    %Derivative coefficients
    dco=sf(ones(npy,1),:).*co(:,1:nco-1);
    
    %Build pp form for derivative
    ppd=mkpp(br,dco);
    
    %Generate the values of dy_by_dx by evaluating the piecewise polynomial
    dy_by_dx = ppval(ppd,x);
    
else
    dy_by_dx = NaN(size(x));
end

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code
