%plot_data
% Function to plot graphs from Excel analysis with 'zero' data points ignored.

function plot_data

%Define number of interpolation points
k.N = 1000;

%Define fontsize for plots
fsize = 14;

%%%%%%%%%%%%

%Open Excel file
[filename, pathname] = uigetfile( '*.xls','Choose Excel file');
if filename == 0
    return
end
[num,txt,raw] = xlsread([pathname,filename]);
dim = size(raw);

%Assign numerical data
k.frame = num(:,1).';
k.t = num(:,2).';
k.x = num(:,3).';
k.y = num(:,4).';

%Update kinematics from x,y,t data
k = update_kinematics( k.x, k.y, k.t, k.N );

%Contruct x,y vs t plots
figure;
plot( k.ti,k.xi,'r' ); hold on;
plot( k.t,k.x,'r+' );
plot( k.ti,k.yi,'b' );
plot( k.t,k.y,'b+' );
title({filename,'x and y vs t'},'fontsize',fsize);
xlabel('t /s','fontsize',fsize);
ylabel('displacement /metres','fontsize',fsize);
set(gca,'fontsize',fsize);
legend({'x','xi','y','yi'},'fontsize',fsize)
grid on;
print( gcf, [pathname,filename,' x vs y vs t.png'],'-dpng','-r300' );
clf;

%Construct vx,vy vs t plots
plot( k.ti,k.vxi,'r' ); hold on;
plot( k.t,k.vx,'r+' );
plot( k.ti,k.vyi,'b' );
plot( k.t,k.vy,'b+' );
title({filename,'vx and vy vs t'},'fontsize',fsize);
xlabel('t /s','fontsize',fsize);
ylabel('velocity /ms^{-1}','fontsize',fsize);
set(gca,'fontsize',fsize);
legend({'vx','vxi','vy','vyi'},'fontsize',fsize)
grid on;
print( gcf, [pathname,filename,' vx vs vy.png'],'-dpng','-r300' );
clf;

%Contruct ax,ay vs t plots
plot( k.ti,k.axi,'r' ); hold on;
plot( k.t,k.ax,'r+' );
plot( k.ti,k.ayi,'b' );
plot( k.t,k.ay,'b+' );
title({filename,'ax and ay vs t'},'fontsize',fsize);
xlabel('t /s','fontsize',fsize);
ylabel('acceleration /ms^{-2}','fontsize',fsize);
set(gca,'fontsize',fsize);
legend({'ax','axi','ay','ayi'},'fontsize',fsize)
grid on;
print( gcf, [pathname,filename,' ax vs ay.png'],'-dpng','-r300' );
clf;

%Contruct x vs yplots
plot( k.xi,k.yi,'r' ); hold on;
plot( k.x,k.y,'r+' );
plot( k.x(1),k.y(1),'g*' );
plot( k.x(end),k.y(end),'m*' );
axis equal
title({filename,'x vs y'},'fontsize',fsize);
xlabel('x /metres','fontsize',fsize);
ylabel('y /metres','fontsize',fsize);
set(gca,'fontsize',fsize);
grid on;
print( gcf, [pathname,filename,' x vs y.png'],'-dpng','-r300' );
close(gcf);

%%

%Update kinematics, based upon vectors of x, y and time (t) coordinates
%corresponding to the centre of the red blob for each frame. N is the
%number of data points for an interpolated set of kinematic parameters.
function k = update_kinematics( x, y, t, N )

%Store original data in structure k
k.t = t;
k.x = x;
k.y = y;
k.N = N;

%Interpolate x,y,t
k.ti = linspace( t(1),t(end),N);
k.xi = interp1( t,x, k.ti, 'spline' );
k.yi = interp1( t,y, k.ti, 'spline' );

%Differentiate x,y data to form velocities and accelerations
k.vx = differentiator( t, x, t );
k.vy = differentiator( t, y, t );
k.ax = differentiator( t, k.vx, t );
k.ay = differentiator( t, k.vy, t );

%Interpolate velocities and accelerations to enable a smooth plot to be
%formed
k.vxi = interp1( t,k.vx, k.ti, 'spline' );
k.vyi = interp1( t,k.vy, k.ti, 'spline' );
k.axi = interp1( t,k.ax, k.ti, 'spline' );
k.ayi = interp1( t,k.ay, k.ti, 'spline' );

%%

%differentiator
% Differentiation function using cubic splines.
%
% Syntax: dy_by_dx = differentiate( xx, yy, x )
%
% xx, yy     Samples from function y(x) which is being differentiated. These
%            must be the same dimensions and have at least three elements each.
% x          Point on y(x) at which the gradient is returned. This can be an array of any
%            dimensions.
%
% dy_by_dx   Output of gradient of y(x) corresponindg to array x
%
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.

function dy_by_dx = differentiator( xx, yy, x )

%Check inputs are the right size
if ( numel(xx) == numel(yy) ) && ( numel(xx) >= 3 )
    
    %Sort xx in ascending order
    [xx,i] = sort(xx);
    yy = yy(i);
    
    %Compute piecewise polynomial based upon cubic splines.
    pp=spline(xx,yy);
    %pp = pchip(xx,yy);  %Alternative Hermite polynomial fit. Apparently this
    %is slightly better at preserving the shape of the function near
    %discontinuities.
    
    %Take apart piecewise polynomial
    [br,co,npy,nco]=unmkpp(pp);
    
    %Scale factors for differentiation
    sf=nco-1:-1:1;
    
    %Derivative coefficients
    dco=sf(ones(npy,1),:).*co(:,1:nco-1);
    
    %Build pp form for derivative
    ppd=mkpp(br,dco);
    
    %Generate the values of dy_by_dx by evaluating the piecewise polynomial
    dy_by_dx = ppval(ppd,x);
    
else
    dy_by_dx = NaN(size(x));
end

%End of code