% SoundAnalyser
% Graphical user interface to a sound recording analysis tool.
% * Records sound for a specified duration using the current audio input
% (e.g. microphone etc).
% * Alternatively an MP3 or WAV file can be loaded
% * Waveform (amplitude vs time )for L and R audio channels is plotted
% * A spectrogram (frequency content vs time in a coloured surface) is
%   generated.
% * A MATLAB .mat file can be exported of the audio data, spectrogram etc
% * High quality PNG prints of the graphs can be created.
%
% LAST UPDATED by Andy French December 2019

%%%% GUI INITIALIZATION %%%%

%Top level GUI initialization
function varargout = SoundAnalyser(varargin)

% Generic MATLAB GUI initialization. Edit at your peril!
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @SoundAnalyser_OpeningFcn, ...
    'gui_OutputFcn',  @SoundAnalyser_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

% Outputs from this function are returned to the command line.
function varargout = SoundAnalyser_OutputFcn(hObject, eventdata, handles)
varargout{1} = handles.output;

%%

% Code which executes just before SoundAnalyser is made visible.
function SoundAnalyser_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for SoundAnalyser
handles.output = hObject;
guidata(hObject, handles);

%Add all subdirectories within current directory to MATLAB path
addpath( genpath(pwd), '-begin');

%Store GUI object handles data in a global array d which can be seen (and
%modified) by all functions. This will be the main GUI data array which
%enables all objects to communicate with one another.
global d
d.h = handles; clear handles

%Centre figure
figcentre(d.h.FIGURESoundAnalyser);

%Set default name for GUI
d.name = 'SoundAnalyser';
set( d.h.FIGURESoundAnalyser,'name',d.name);

%Initially turn off axis
set( d.h.AXESwaveformLEFT,'visible','off');
set( d.h.AXESwaveformRIGHT,'visible','off');
set( d.h.AXESspectrogram,'visible','off');

%Initialize sound player parameters
d.music.play = 0;
d.music.reverse = 0;
d.music.filename = '';
d.music.audioplayer_obj = [];

%Initialize sound recorder parameters
d.music.recording_duration_s = 5;
d.music.recording_countdown_s = 5;
d.music.audiorecorder_obj = [];

%Initialize sound analysis parameters

%Waveform plot
d.sound.waveform_left_ch = [];
d.sound.waveform_right_ch = [];
d.sound.waveform_t_s = [];
d.sound.waveform_sample_rate_Hz = [];
d.sound.waveform_duration_s = [];
d.sound.waveform_tmin = 0;
d.sound.waveform_tmax = d.music.recording_duration_s;
d.sound.waveform_ymin = [];
d.sound.waveform_ymax = [];

%Spectrogram plot
d.sound.spectrogram_freq_Hz = [];
d.sound.spectrogram_t_s = [];
d.sound.spectrogram = [];
d.sound.spectrogram_dominant_freq_Hz = [];
d.sound.spectrogram_delta_f = 10;
d.sound.spectrogram_STFFT_duration_s = [];
d.sound.spectrogram_smooth_factor = 0.5;
d.sound.spectrogram_tmin = 0;
d.sound.spectrogram_tmax = d.music.recording_duration_s;
d.sound.spectrogram_fmin = 0;
d.sound.spectrogram_fmax = 10*1000;


%Initialize GUI parameters for music aspects
set( d.h.TOGGLEplay,'value',0 );
set( d.h.TOGGLEreverse,'value',0 );
set( d.h.EDITcountdown, 'string', num2str( d.music.recording_countdown_s ) );
set( d.h.EDITcountdown, 'string', num2str( d.music.recording_duration_s ) );
set( d.h.EDITwaveformtmin, 'string', num2str( d.sound.waveform_tmin ) );
set( d.h.EDITwaveformtmax, 'string', num2str( d.sound.waveform_tmin ) );
set( d.h.EDITspectrogramtmin, 'string', num2str( d.sound.spectrogram_tmin ) );
set( d.h.EDITspectrogramtmax, 'string', num2str( d.sound.spectrogram_tmax ) );
set( d.h.EDITspectrogramfmin, 'string', num2str( d.sound.spectrogram_fmin ) );
set( d.h.EDITspectrogramfmax, 'string', num2str( d.sound.spectrogram_fmax ) );
set( d.h.EDITspectrogramdeltaf, 'string', num2str( d.sound.spectrogram_delta_f ) );

%Set visibility of graph scale edit boxes
set( d.h.EDITwaveformtmin,'visible','off');
set( d.h.EDITwaveformtmax, 'visible','off');
set( d.h.EDITwaveformymin, 'visible','off');
set( d.h.EDITwaveformymax, 'visible','off');
set( d.h.EDITspectrogramtmin, 'visible','off' );
set( d.h.EDITspectrogramtmax, 'visible','off' );
set( d.h.EDITspectrogramfmin, 'visible','off' );
set( d.h.EDITspectrogramfmax, 'visible','off' );
set( d.h.EDITspectrogramdeltaf, 'visible','off' );
set( d.h.EDITspectrogramdeltat, 'visible','off' );
set( d.h.TEXTwaveformtmin,'visible','off');
set( d.h.TEXTwaveformtmax, 'visible','off');
set( d.h.TEXTwaveformymin, 'visible','off');
set( d.h.TEXTwaveformymax, 'visible','off');
set( d.h.TEXTspectrogramtmin, 'visible','off' );
set( d.h.TEXTspectrogramtmax, 'visible','off' );
set( d.h.TEXTspectrogramfmin, 'visible','off' );
set( d.h.TEXTspectrogramfmax, 'visible','off' );
set( d.h.TEXTspectrogramdeltaf, 'visible','off' );
set( d.h.TEXTspectrogramdeltat, 'visible','off' );

%%

%%% GUI CALLBACK FUNCTIONS %%%%

% Pushbutton which loads a .mp3 or .wav music file. This brings up the
% standard windows file window. The filename of the selected file is
% displayed in the GUI and the file contents stored in d.music.music
function PUSHloadmusic_Callback(hObject, eventdata, handles)
global d

%Bring up a windows file browser
[filename, pathname] = uigetfile({'*.mp3;*.wav'}, 'Choose .mp3 or .wav file to load');
if filename ~=0
    
    %Clear any previous audioplayer object
    if ~isempty( d.music.audioplayer_obj )
        stop( d.music.audioplayer_obj );
        delete(d.music.audioplayer_obj)
    end
    
    %Store the filename in the GUI data
    d.music.filename = [pathname,filename];
    
    %Update the edit box with the filename only
    set( d.h.EDITloadmusic, 'string', filename );
    
    %Start clock and display loading message to the infobar
    t0 = tic;
    set( d.h.FIGURESoundAnalyser,'name',['Loading file: ',d.music.filename,' ...'] );
    drawnow;
    
    %Load music file
    if strcmp( filename( end-3 : end ), '.mp3' )
        [s,d.sound.waveform_sample_rate_Hz,Nbits] = mp3read( d.music.filename );
    elseif strcmp( filename( end-3 : end ), '.wav' )
        if exist('audioread.m')==0
            [s,d.sound.waveform_sample_rate_Hz,Nbits] = wavread( d.music.filename );
        else
            [s,d.sound.waveform_sample_rate_Hz] = audioread( d.music.filename );
            info = audioinfo(filename);
            Nbits = info.BitsPerSample;
        end
    end
    
    %Convert to Stereo if .wav file is Mono
    dim = size(s);
    if dim(2)==1
        %Mono recording
        d.sound.waveform_left_ch = s;
        d.sound.waveform_right_ch = s;
    else
        %Stereo recording (ignore any other channels!)
        d.sound.waveform_left_ch = s(:,1);
        d.sound.waveform_right_ch = s(:,2);
    end
    
    %Create audioplayer object from music file
    m = [d.sound.waveform_left_ch,d.sound.waveform_right_ch];
    if d.music.reverse == 1;
        %Reverse music if option set
        m = flipdim(m,1);
    end
    delete( d.music.audioplayer_obj );
    d.music.audioplayer_obj = audioplayer( m, d.sound.waveform_sample_rate_Hz );
    play( d.music.audioplayer_obj );
    pause( d.music.audioplayer_obj );
    
    %Play is option set
    if d.music.play == 1
        resume( d.music.audioplayer_obj );
    end
    
    %Display loading completion message to info bar
    set( d.h.FIGURESoundAnalyser,'name',...
        ['File: ',d.music.filename,' loaded in ',num2str(toc(t0)),' seconds.'] );
    
    %Analyse waveform
    [ d.sound.waveform_t_s, d.sound.waveform_duration_s,...
        d.sound.spectrogram_dominant_freq_Hz, d.sound.spectrogram,...
        d.sound.spectrogram_freq_Hz, d.sound.spectrogram_t_s, d.sound.spectrogram_STFFT_duration_s  ] = ...
        wavanalyser(  m, d.sound.waveform_sample_rate_Hz,...
        d.sound.spectrogram_smooth_factor,...
        d.sound.spectrogram_delta_f );
    
    %Update max and min waveform scale to fit data
    d.sound.waveform_ymin = min( min(m) );
    d.sound.waveform_ymax = max( max(m) );
    clear m
    
    %Update axes scale edit boxes
    set( d.h.EDITwaveformymin,'string',num2str( d.sound.waveform_ymin ));
    set( d.h.EDITwaveformymax,'string',num2str( d.sound.waveform_ymax ));
    d.sound.waveform_tmin = 0;
    d.sound.waveform_tmax = d.sound.waveform_duration_s;
    d.sound.spectrogram_tmin = 0;
    d.sound.spectrogram_tmax = d.sound.waveform_duration_s;
    set( d.h.EDITwaveformtmin,'string','0' );
    set( d.h.EDITwaveformtmax,'string',num2str( d.sound.waveform_duration_s ) );
    set( d.h.EDITspectrogramtmin,'string','0' );
    set( d.h.EDITspectrogramtmax,'string',num2str( d.sound.waveform_duration_s ) );
    
    %Update plots
    update_plots;
end

%%

%Load music file into the default Windows program
function PUSHmusicopen_Callback(hObject, eventdata, handles)
global d

%Open file using default windows program
if strcmp( d.music.filename,'') ~=1
    winopen( d.music.filename );
end

%%

%Toggle button which plays any music loaded. It toggles between a play and
%pause status.
function TOGGLEplay_Callback(hObject, eventdata, handles)
global d

%Manage play or not play toggle status
if d.music.play == 1
    d.music.play = 0;
    set( d.h.TOGGLEplay,'value',0 );
    set( d.h.TOGGLEplay,'BackGroundColor',[0.941,0.941,0.941] );
    if ~isempty( d.music.audioplayer_obj )
        pause( d.music.audioplayer_obj );
    end
else
    
    %Play music
    d.music.play = 1;
    set( d.h.TOGGLEplay,'value',1 );
    set( d.h.TOGGLEplay,'BackGroundColor',[0,1,0] );
    
    %Check if any music has been loaded
    if ~isempty( d.music.audioplayer_obj )
        resume( d.music.audioplayer_obj );
    end
end

%%

%Toggle button which regenerates an audioplayer object, but using the sound
%samples in d.music.music played in reverse!
function TOGGLEreverse_Callback(hObject, eventdata, handles)
global d

%Manage reverse or not reverse toggle status
if d.music.reverse == 1
    d.music.reverse = 0;
    set( d.h.TOGGLEreverse,'value',0 );
    set( d.h.TOGGLEreverse,'BackGroundColor',[0.941,0.941,0.941] );
else
    d.music.reverse = 1;
    set( d.h.TOGGLEreverse,'value',1 );
    set( d.h.TOGGLEreverse,'BackGroundColor',[1,0.1,0.1] );
end

%If some music has already been loaded ....
if ~isempty( d.music.audioplayer_obj )
    
    %Clear any previous audioplayer object
    stop( d.music.audioplayer_obj );
    delete(d.music.audioplayer_obj)
    
    %Create audioplayer object from music file;
    m = [d.sound.waveform_left_ch,d.sound.waveform_right_ch];
    if d.music.reverse == 1;
        %Reverse music if option set
        m = flipdim(m,1);
    end
    
    %Recreate audioplayer object
    d.music.audioplayer_obj = audioplayer( m, d.sound.waveform_sample_rate_Hz );
    play( d.music.audioplayer_obj );
    pause( d.music.audioplayer_obj );
    
    %Play is option set
    if d.music.play == 1
        resume( d.music.audioplayer_obj );
    end
end

%%

%Music restart pushbutton. This stops the current audioplayer object
%and then, if the play toggle is pressed, plays the music in d.music.music
%from the beginning.
function PUSHrestart_Callback(hObject, eventdata, handles)
global d
if ~isempty( d.music.audioplayer_obj )
    stop( d.music.audioplayer_obj );
    if d.music.play == 1;
        play( d.music.audioplayer_obj );
    end
end

%%

%Pushbutton which initiates a sound recording (following a countdown with GUI feedback). 
%The sound recording is then played back.
function PUSHrecord_Callback(hObject, eventdata, handles)
global d

%Number of channels and sample rate
nChannels = 2;
d.sound.waveform_sample_rate_Hz = 44100;

%Can the audio input device support a higher bit depth?
if audiodevinfo(1,0,44100,24,2)==1;
    nBits = 24;
else
    nBits = 16;
end

%

%Change colour of push button and initiate countdown
set( d.h.PUSHrecord,'backgroundcolor', [1,0,0] );
set( d.h.FIGURESoundAnalyser,'name',...
    [ 'Recording in ',num2str(d.music.recording_countdown_s), ' seconds ...' ] );
set( d.h.PUSHrecord,'string',num2str(ceil( d.music.recording_countdown_s )), 'fontsize', 100 );
for n=1:ceil(d.music.recording_countdown_s)
    pause(1);
    set( d.h.PUSHrecord,'string',num2str(ceil( d.music.recording_countdown_s ) - n), 'fontsize', 100 );
    set( d.h.FIGURESoundAnalyser,'name',...
        [ 'Recording in ',num2str(ceil( d.music.recording_countdown_s ) - n), ' seconds ...' ] );
end

%Display 'RECORDING...' message
set( d.h.PUSHrecord,'string','RECORDING', 'fontsize', 32  );
set( d.h.FIGURESoundAnalyser,'name',...
    ['RECORDING for ',num2str( d.music.recording_duration_s ),' seconds....'] );

%Create audiorecorder object
d.music.audiorecorder_obj = audiorecorder( d.sound.waveform_sample_rate_Hz,nBits,nChannels );

%Make a recording using the deault sound recorder system (e.g. webcam,
%microphone in etc)
recordblocking( d.music.audiorecorder_obj, d.music.recording_duration_s );

%Restore string of pushbutton and background colour
set( d.h.PUSHrecord, 'string', 'Record','fontsize',32 );
set( d.h.PUSHrecord,'backgroundcolor', [0,1,0] );
set( d.h.FIGURESoundAnalyser,'name','Recording complete.' );

%Store data in double-precision array.
m = getaudiodata(d.music.audiorecorder_obj);
d.sound.waveform_left_ch = m(:,1);
d.sound.waveform_right_ch = m(:,2);

%Generate audioplayer object from sound smaples
delete( d.music.audioplayer_obj );
d.music.audioplayer_obj = audioplayer( m, d.sound.waveform_sample_rate_Hz );
play( d.music.audioplayer_obj );

%Analyse recorded waveform
[ d.sound.waveform_t_s, d.sound.waveform_duration_s,...
    d.sound.spectrogram_dominant_freq_Hz, d.sound.spectrogram,...
    d.sound.spectrogram_freq_Hz, d.sound.spectrogram_t_s, d.sound.spectrogram_STFFT_duration_s  ] = ...
    wavanalyser(  m, d.sound.waveform_sample_rate_Hz,...
    d.sound.spectrogram_smooth_factor,...
    d.sound.spectrogram_delta_f );

%Update max and min waveform scale to fit data
d.sound.waveform_ymin = min( min(m) );
d.sound.waveform_ymax = max( max(m) );
clear m

%Update axes scale edit boxes
set( d.h.EDITwaveformymin,'string',num2str( d.sound.waveform_ymin ));
set( d.h.EDITwaveformymax,'string',num2str( d.sound.waveform_ymax ));
d.sound.waveform_tmin = 0;
d.sound.waveform_tmax = d.sound.waveform_duration_s;
d.sound.spectrogram_tmin = 0;
d.sound.spectrogram_tmax = d.sound.waveform_duration_s;
set( d.h.EDITwaveformtmin,'string','0' );
set( d.h.EDITwaveformtmax,'string',num2str( d.sound.waveform_duration_s ) );
set( d.h.EDITspectrogramtmin,'string','0' );
set( d.h.EDITspectrogramtmax,'string',num2str( d.sound.waveform_duration_s ) );

%Update music file loading box
d.music.filename = '';
set( d.h.EDITloadmusic, 'string', '' );

%Update plots
update_plots;


%%

%Export .PNG file for graphs
function PUSHexportPNG_Callback(hObject, eventdata, handles)
global d
if ~isempty( d.sound.waveform_right_ch )
    
    %Construct filename(s)
    nowstr = strrep( datestr(now), ':','-' );
    waveforms_filename = [ 'waveform ',nowstr ];
    spectrogram_filename = [ 'spectrogram ',nowstr ];
    dom_freq_filename = [ 'dominant frequencies ',nowstr ];
    
    %Waveform plot
    figure('color',[1 1 1] );
    fsize = 16;
    
    %Left channel
    subplot(2,1,1);
    plot( d.sound.waveform_t_s, d.sound.waveform_left_ch.', 'r' );
    ylim( [d.sound.waveform_ymin,d.sound.waveform_ymax] );
    xlim( [d.sound.waveform_tmin,d.sound.waveform_tmax] );
    xlabel('time /s','fontsize',fsize)
    ylabel('Waveform signal','fontsize',fsize)
    title('Waveform signal vs time: Left channel','fontsize',fsize);
    set(gca,'fontsize',fsize);
    grid on
    
    %Right channel
    subplot(2,1,2);
    plot( d.sound.waveform_t_s, d.sound.waveform_right_ch.', 'b' );
    ylim( [d.sound.waveform_ymin,d.sound.waveform_ymax] );
    xlim( [d.sound.waveform_tmin,d.sound.waveform_tmax] );
    xlabel('time /s','fontsize',fsize)
    ylabel('Waveform signal','fontsize',fsize)
    title('Waveform signal vs time: Right channel','fontsize',fsize);
    set(gca,'fontsize',fsize);
    grid on
    
    %Print PNG of waveforms
    print( gcf, waveforms_filename, '-dpng', '-r300' );
    
    %Spectrogram plot
    fsize = 14;
    clf; colormap('jet');
    pcolor(d.sound.spectrogram_t_s,d.sound.spectrogram_freq_Hz/1000,d.sound.spectrogram.');
    caxis([-80,0]); view(2);
    hold on; 
    plot( d.sound.waveform_t_s, d.sound.spectrogram_dominant_freq_Hz/1000, 'm-','linewidth',0.5 );
    shading flat
    ylim([d.sound.spectrogram_fmin/1000,d.sound.spectrogram_fmax/1000]);
    xlim([d.sound.spectrogram_tmin,d.sound.spectrogram_tmax]);
    interp_colormap(1000); colorbar('location','Southoutside','fontsize',fsize)
    xlabel('Time /s','fontsize',fsize); ylabel('Frequency /kHz','fontsize',fsize)
    set(gca,'fontsize',fsize);
    title('Normalized spectrogram /dB: Frequency spectrum variation with time','fontsize',fsize);
    print( gcf, spectrogram_filename, '-dpng', '-r300' );
    
    %Dominant frequency plot
    clf;
    fsize = 18;
    plot( d.sound.waveform_t_s, d.sound.spectrogram_dominant_freq_Hz/1000, 'm' );
    axis tight; xlim([d.sound.spectrogram_tmin,d.sound.spectrogram_tmax]);
    xlabel('Time /s','fontsize',fsize); ylabel('Frequency /kHz','fontsize',fsize); grid on
    title('Dominant frequency component vs time','fontsize',fsize)
    set(gca,'fontsize',fsize);
    print( gcf, dom_freq_filename, '-dpng', '-r300' );
    close(gcf);
end

%%

%Export .mat file for sound data
function PUSHexportmat_Callback(hObject, eventdata, handles)
global d

%Construct filename
filename = ['sound_data ',strrep( datestr(now),':','-' )];

%Export .mat file
s = d.sound;
save( filename, '-struct', 's' );
clear s

%%

%Export .mp3 file for sound data
function PUSHexportmp3_Callback(hObject, eventdata, handles)
global d

if ~isempty( d.sound.waveform_right_ch )
    %Construct filename
    filename = ['soundfile ',strrep( datestr(now),':','-' )];  
    %Export .mp3
    mp3write( [d.sound.waveform_left_ch,d.sound.waveform_right_ch],...
        d.sound.waveform_sample_rate_Hz, filename );
end

%%

%Recording countdown edit box
function EDITcountdown_Callback(hObject, eventdata, handles)
global d
d.music.recording_countdown_s = str2num( get( d.h.EDITcountdown, 'string' ) );
if d.music.recording_countdown_s <=0
    d.music.recording_countdown_s = 5;
    get( d.h.EDITcountdown, 'string', '5' );
end

%%

%Recording duration edit box
function EDITduration_Callback(hObject, eventdata, handles)
global d
d.music.recording_duration_s = str2num( get( d.h.EDITduration, 'string' ) );
if d.music.recording_duration_s <=0
    d.music.recording_duration_s = 5;
    get( d.h.EDITduration, 'string', '5' );
end

%%

%Waveform graph tmin editbox
function EDITwaveformtmin_Callback(hObject, eventdata, handles)
global d
t = str2num( get( d.h.EDITwaveformtmin, 'string' ) );
if ( t < d.sound.waveform_tmax ) && ( t >=0 )
    d.sound.waveform_tmin = t;
else
    set( d.h.EDITwaveformtmin, 'string', d.sound.waveform_tmin )
end

%Get y max and min
i = find( ( d.sound.waveform_t_s >= d.sound.waveform_tmin ) &...
    ( d.sound.waveform_t_s <= d.sound.waveform_tmax ) );
left = d.sound.waveform_left_ch(i);
right = d.sound.waveform_right_ch(i);
d.sound.waveform_ymin = min( [min(left),min(right) ] );
d.sound.waveform_ymax = max( [max(left),max(right) ] );
update_plots

%%

%Waveform graph tmax editbox
function EDITwaveformtmax_Callback(hObject, eventdata, handles)
global d
t = str2num( get( d.h.EDITwaveformtmax, 'string' ) );
if ( t > d.sound.waveform_tmin )
    d.sound.waveform_tmax = t;
else
    set( d.h.EDITwaveformtmax, 'string', d.sound.waveform_tmax )
end

%Get y max and min
i = find( ( d.sound.waveform_t_s >= d.sound.waveform_tmin ) &...
    ( d.sound.waveform_t_s <= d.sound.waveform_tmax ) );
left = d.sound.waveform_left_ch(i);
right = d.sound.waveform_right_ch(i);
d.sound.waveform_ymin = min( [min(left),min(right) ] );
d.sound.waveform_ymax = max( [max(left),max(right) ] );
update_plots

%%

%Waveform graph ymin editbox
function EDITwaveformymin_Callback(hObject, eventdata, handles)
global d
y = str2num( get( d.h.EDITwaveformymin, 'string' ) );
if ( y < d.sound.waveform_ymax )
    d.sound.waveform_ymin = y;
else
    set( d.h.EDITwaveformymin, 'string', d.sound.waveform_ymin )
end
update_plots

%%

%Waveform graph ymax editbox
function EDITwaveformymax_Callback(hObject, eventdata, handles)
global d
y = str2num( get( d.h.EDITwaveformymax, 'string' ) );
if ( y > d.sound.waveform_ymin )
    d.sound.waveform_ymax = y;
else
    set( d.h.EDITwaveformymax, 'string', d.sound.waveform_ymax )
end
update_plots

%%

%Spectrogram tmin editbox
function EDITspectrogramtmin_Callback(hObject, eventdata, handles)
global d
t = str2num( get( d.h.EDITspectrogramtmin, 'string' ) );
if ( t < d.sound.spectrogram_tmax ) && ( t>=0 )
    d.sound.spectrogram_tmin = t;
else
    set( d.h.EDITspectrogramtmin, 'string', d.sound.spectrogram_tmin )
end
update_plots

%%

%Spectrogram tmax editbox
function EDITspectrogramtmax_Callback(hObject, eventdata, handles)
global d
t = str2num( get( d.h.EDITspectrogramtmax, 'string' ) );
if ( t > d.sound.spectrogram_tmin )
    d.sound.spectrogram_tmax = t;
else
    set( d.h.EDITspectrogramtmax, 'string', d.sound.spectrogram_tmax )
end
update_plots

%%

%Spectrogram fmin editbox
function EDITspectrogramfmin_Callback(hObject, eventdata, handles)
global d
f = str2num( get( d.h.EDITspectrogramfmin, 'string' ) );
if ( f < d.sound.spectrogram_fmax ) && ( f>=0 )
    d.sound.spectrogram_fmin = f;
else
    set( d.h.EDITspectrogramfmin, 'string', d.sound.spectrogram_fmin )
end
update_plots

%%

%Spectrogram fmax editbox
function EDITspectrogramfmax_Callback(hObject, eventdata, handles)
global d
f = str2num( get( d.h.EDITspectrogramfmax, 'string' ) );
if ( f > d.sound.spectrogram_fmin )
    d.sound.spectrogram_fmax = f;
else
    set( d.h.EDITspectrogramfmax, 'string', d.sound.spectrogram_fmax )
end
update_plots

%%

%Spectrogram deltaf editbox
function EDITspectrogramdeltaf_Callback(hObject, eventdata, handles)
global d
d.sound.spectrogram_delta_f = str2num( get( d.h.EDITspectrogramdeltaf, 'string' ) );
if d.sound.spectrogram_delta_f < 0
    %Default frequency resolution /Hz
    d.sound.spectrogram_delta_f = 10;
    set( d.h.EDITspectrogramdeltaf, 'string', '10' )  
end

%Analyse recorded waveform
[ d.sound.waveform_t_s, d.sound.waveform_duration_s,...
    d.sound.spectrogram_dominant_freq_Hz, d.sound.spectrogram,...
    d.sound.spectrogram_freq_Hz, d.sound.spectrogram_t_s, d.sound.spectrogram_STFFT_duration_s  ] = ...
    wavanalyser(  [d.sound.waveform_left_ch,d.sound.waveform_right_ch],...
    d.sound.waveform_sample_rate_Hz,...
    d.sound.spectrogram_smooth_factor,...
    d.sound.spectrogram_delta_f );

%Update plots
update_plots

%%

%GUI close function
function closerequestfunc(hObject, eventdata, handles)
global d

%Close GUI
delete( d.h.FIGURESoundAnalyser );

%Stop any music playing and clear any previous audioplayer object
if ~isempty( d.music.audioplayer_obj )
    stop( d.music.audioplayer_obj );
    delete(d.music.audioplayer_obj)
end

%Remove sub-directories within current directory from MATLAB path
rmpath( genpath(pwd) );

%%

%Waveform analysis function
function [ waveform_t_s, duration_s,...
    spectrogram_dominant_freq_Hz, S, spectrogram_freq_Hz, spectrogram_t_s, delta_t  ] = ...
    wavanalyser(  s, waveform_sample_rate_Hz, smooth_factor, delta_f )

%Combine sound channels and convert to row vector
s = sum(s,2).'; N = length(s);

%Duration of recording /s
duration_s = (numel(s)-1)/waveform_sample_rate_Hz;

%

% COMPUTE SHORT TIME FOURIER TRANSFORM %

%Determine number of samples which corresponds to STFFT
num_in_STFFT = round( waveform_sample_rate_Hz / delta_f );

%Determine number of STFFTs
num_STFFTs = ceil( length(s) / num_in_STFFT );

%Pad signal with zeros to make sure it has total number of samples =
%num_s_in_STFFT x num_STFFTs
if length(s) < num_STFFTs * num_in_STFFT
    dim = size(s);
    s = [ s, zeros(1, num_STFFTs * num_in_STFFT - N ) ];
end

%Determine original time sample array
waveform_t_s = linspace(0, duration_s, N );

%Transform interpolated y vector into a matrix of M rows * 2*M columns
S = reshape(s,[num_in_STFFT,num_STFFTs]);

%Compute weighted STFT over rows for each column. Output result in
%decibels.
w = dolphcheb(num_in_STFFT,40);
w = repmat(w.',[1,num_STFFTs]);
S = dB( fft( w.*S ,[],1 ) );

%Ignore -ve frequencies. Since signal only contains amplitude information
%it will be symmetric. Assume no frequencies are dominant which are over
%half the sample rate!
S = S(1:round(num_in_STFFT/2),:);
[rows,cols] = size(S);

%Define fractional time and frequencies for spectrogram
spectrogram_freq_Hz = linspace(0, waveform_sample_rate_Hz/2, rows );
spectrogram_t_s = linspace(0,duration_s,cols);
delta_t = num_in_STFFT/waveform_sample_rate_Hz;
[spectrogram_freq_Hz,spectrogram_t_s]=meshgrid(spectrogram_freq_Hz,spectrogram_t_s);

%

% SMOOTH SIGNAL AND FIND DOMINANT FREQUENCIES

%Compute dominant frequency /Hz for each time interval
spectrogram_dominant_freq_Hz = zeros(1,num_STFFTs);
FF = linspace(0, waveform_sample_rate_Hz/2, round(num_in_STFFT/2) );
fd=[];
for n=1:num_STFFTs
    fd = FF(  S(:,n) == max( S(:,n) ) ) ;
    if ~isempty(fd)
        spectrogram_dominant_freq_Hz(n) = fd(1);
    end
end
clear fd FF

%Interpolate at desired frame rate
spectrogram_dominant_freq_Hz =...
    interp1( linspace( 0, duration_s, length(spectrogram_dominant_freq_Hz) ),...
    spectrogram_dominant_freq_Hz, waveform_t_s );

%Apply smoothing
spectrogram_dominant_freq_Hz = smooth( spectrogram_dominant_freq_Hz,smooth_factor );

%%

%Update waveform and spectrogram plots
function update_plots
global d
if ~isempty( d.sound.waveform_right_ch );
    
    %Set visibility of graph scale edit boxes
    set( d.h.EDITwaveformtmin,'visible','on');
    set( d.h.EDITwaveformtmax, 'visible','on');
    set( d.h.EDITwaveformymin, 'visible','on');
    set( d.h.EDITwaveformymax, 'visible','on');
    set( d.h.EDITspectrogramtmin, 'visible','on' );
    set( d.h.EDITspectrogramtmax, 'visible','on' );
    set( d.h.EDITspectrogramfmin, 'visible','on' );
    set( d.h.EDITspectrogramfmax, 'visible','on' );
    set( d.h.EDITspectrogramdeltaf, 'visible','on' );
    set( d.h.EDITspectrogramdeltat, 'visible','on',...
        'string',num2str( 1000*d.sound.spectrogram_STFFT_duration_s)  );
    set( d.h.TEXTwaveformtmin,'visible','on');
    set( d.h.TEXTwaveformtmax, 'visible','on');
    set( d.h.TEXTwaveformymin, 'visible','on');
    set( d.h.TEXTwaveformymax, 'visible','on');
    set( d.h.TEXTspectrogramtmin, 'visible','on' );
    set( d.h.TEXTspectrogramtmax, 'visible','on' );
    set( d.h.TEXTspectrogramfmin, 'visible','on' );
    set( d.h.TEXTspectrogramfmax, 'visible','on' );
    set( d.h.TEXTspectrogramdeltaf, 'visible','on' );
    set( d.h.TEXTspectrogramdeltat, 'visible','on');
    
    %Make axis visible
    set( d.h.AXESwaveformLEFT,'visible','on');
    set( d.h.AXESwaveformRIGHT,'visible','on');
    set( d.h.AXESspectrogram,'visible','on');
    
    %Waveform plot
    
    %Left channel
    axes( d.h.AXESwaveformLEFT ); cla;
    plot( d.sound.waveform_t_s, d.sound.waveform_left_ch.', 'r' );
    ylim( [d.sound.waveform_ymin,d.sound.waveform_ymax] );
    set( gca, 'ytick',[], 'xtick',[] );
    xlim( [d.sound.waveform_tmin,d.sound.waveform_tmax] );
    
    %Right channel
    axes( d.h.AXESwaveformRIGHT ); cla;
    plot( d.sound.waveform_t_s, d.sound.waveform_right_ch.', 'b' );
    xlabel('time /s')
    ylim( [d.sound.waveform_ymin,d.sound.waveform_ymax] );
    set( gca, 'ytick',[] );
    xlim( [d.sound.waveform_tmin,d.sound.waveform_tmax] );
    
    %Spectrogram plot
    axes( d.h.AXESspectrogram ); cla;
    pcolor(d.sound.spectrogram_t_s,d.sound.spectrogram_freq_Hz/1000,d.sound.spectrogram.');
    caxis( [-80,0] ); view(2);
    shading flat
    hold on; plot( d.sound.waveform_t_s, d.sound.spectrogram_dominant_freq_Hz/1000, 'm-','linewidth',0.5 );
    ylim([d.sound.spectrogram_fmin/1000,d.sound.spectrogram_fmax/1000]);
    xlim([d.sound.spectrogram_tmin,d.sound.spectrogram_tmax]);
    interp_colormap(1000);
    colorbar
    ylabel('Frequency /kHz'); xlabel('Time /s');
    title('Normalized spectrogram /dB')
end

%%

%DOLPHCHEB
% Function that computes Dolph-Chebyshev windowing function. The main role
% of the window (as with all windows) is to damp out the effects of the
% Gibbs phenomenon that results from truncation of an infinite series (e.g.
% a Discrete Fourier Transform). Note this window is complex and can
% therefore only really be visualised when the amplitude of the window DFT
% is plotted (in dB).
% N      - Length of window sl_level. This ifference between peak and
%          sidelobes /dB in the DFT of the window.
% w      - Window 'weights'. These are real, positive numbers.
function w=dolphcheb(N,sl_level)
gamma=sl_level/20;
if N>1
    M=N-1;
    alpha=cosh(acosh(10^gamma)/M);
    for n=1:M
        A(n)=((-1)^(n-1))*cos(M*acos(alpha*cos(pi*(n-1)/M)));
    end
    B=fft(A); B(1)=0.5*B(1); B(N)=B(1);
    w=B/max(B);
else
    w=1;
end

%%

%smooth
% Smoothing function.
% x is the input vector of values to smooth
% K is the smoothing constant 0 < K < 1. K=1 implies total smoothing, K=0
% implies no smoothing at all.
function y = smooth( x, K )
y=zeros(size(x));
Y=x(1);
for n=1:length(x)
    y(n) = (1-K) * x(n) + K * Y;
    Y = y(n);
end

%%

%dB
% Function which converts a quantity to decibels i.e. a logarithmic scale.
% The maximum value is taken to be 0 dB.
function y = dB(x)
y = NaN(size(x));
x = abs(x);
maxx = max(max(x));
if maxx > 0
    y(x>0) = 10*log10( x(x>0) ) - 10*log10( maxx );
end

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize'); display_size=get(0,'MonitorPositions');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels'); fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*( screen_size(3)-fig_position(3) ) - screen_size(1);
centre_y = 0.5*( screen_size(4)-fig_position(4) ) - screen_size(2);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap, and Initialise new colormap
map = colormap;
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1)); G = ones(1,dim(1)); B = ones(1,dim(1));
RR = ones(1,N); GG = ones(1,N); BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1); G(:) = map(:,2); B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x ); GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:); new_map(:,2) = GG(:); new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code
