%spherium_lite
%
% Simple command line program which enables a surface defined by a radius
% which is a function of azimuth (a) and elevation (e). This can optionally
% be texture mapped and transparency mapped.
%
% LAST UPDATED by Andy French 16/07/2015

function spherium_lite

%Initialise global data store d which all sub-functions can read and
%write to
global d

%Set high resolution image dimensions
d.high_res_width_pix = 2000;
d.high_res_height_pix = 1000;

%

%% Command line interface %%

% Display welcome message to command line
disp(' ');
disp(' Welcome to SPHERIUM LITE ! ' )
disp(' By Andy French July 2015' )
disp(' ')
disp(' Action keys:')
disp(' q   Quit ')
disp(' t   Toggle transparency')
disp(' y   Load new transparency bitmap')
disp(' p   Print a JPEG image of current figure')
disp(' l   Toggle camlight')
disp(' b   Toggle bitmap texture')
disp(' n   Load new bitmap texture')
disp(' h   High resolution toggle')
disp(' s   Export .stl stereo-lithography file')
disp(' ')
disp(' Type radius of surface as a function of azimuth (a) and elevation (e) ...')

% Ask for mathematical function of azimuth and elevation which sets the
% radius of the surface
d.f = input( ' R = ','s' );

%Special test input!
if strcmp( d.f, '' )
    %Default inputs
    d.f = 'sin(a.*e)';
    disp(' ')
    disp( [' Default option selected ... R = ',d.f] );
    d.width_pix = 400;
    d.height_pix = 200;
    d.transparency_fname = [pwd,'\transparencies\holes.jpg'];
    d.texture_fname = [pwd,'\textures\shapes.jpg'];
    [d.B,d.texture_fname] = get_image( d.width_pix, d.height_pix, d.texture_fname );
    [d.T,d.transparency_fname] = get_image( d.width_pix, d.height_pix, d.transparency_fname );
else
    %Replace mathematical operations with a . to enable matrix elementwise
    %multiplication
    d.f = strrep(d.f,'*','.*');
    d.f = strrep(d.f,'/','./');
    d.f = strrep(d.f,'^','.^');
    
    %Request surface resolution, which will correspond to texture
    %and transparency bitmaps in pixels
    disp(' ')
    disp(' State number of azimuth steps')
    d.width_pix = str2num( input( ' width_pix = ','s' ));
    
    disp(' ')
    disp(' State number of elevation steps')
    d.height_pix = str2num( input( ' height_pix = ','s' ));
    
    %Select default texture bitmap
    disp(' ')
    disp(' Get default texture bitmap')
    [d.B,d.texture_fname] = get_image( d.width_pix, d.height_pix, [] );
    
    %Select default transparency bitmap
    disp(' ')
    disp(' Get default transparency bitmap')
    [d.T,d.transparency_fname] = get_image( d.width_pix, d.height_pix, [] );
end

%Store image height and width
d.orig_width_pix = d.width_pix;
d.orig_height_pix = d.height_pix;

%Set texture and transparency to be initially on
d.transparency = 1;
d.texture = 1;

%Set camlight to be initially on
d.light_effect = 1;

%Set high-resolution to be initially off
d.high_res = 0;

%Initialise surface handle
d.s = [];

%

%% Generate original surface %%

%Create figure
d.fig = figure('name','spherium lite',...
    'renderer','opengl','color',[1,1,1],'InvertHardCopy', 'off','KeyPressFcn',@keypressfunc);

%Generate surface
generate_surface;

%%

%Generate original surface
function generate_surface

%Allow read-write usage of global variable d
global d

%Start clock
t0 = tic;
disp(' ')
disp(' Generating surface ...')

%Create arrays of azimuth and elevation /radians
e = linspace( -pi/2, pi/2, d.height_pix );
a = linspace( -pi, pi, d.width_pix );

%Mesh azi and elev arrays
[a,e] = meshgrid(a,e);

%Determine radius using user-defined function f
eval( ['r = ',d.f,';'] );

%If radius is complex, make NaN which means it won't be plotted
r( imag(r) > 1e-6  ) = NaN;

%Make radius a positive number
%r = abs(r);

%Determine x,y,z matrices corresponding to surface
d.x = r.*cos(e).*cos(a);
d.y = r.*cos(e).*sin(a);
d.z = r.*sin(e);

%Plot surface, after deleting any previous surface
figure( d.fig );
delete(d.s);
d.s = plot_surface(d.x,d.y,d.z, d.texture, d.transparency, d.B, d.T, d.light_effect );

%Stop clock and display rendering time
disp(' ')
disp([ ' Surface generated in ',num2str(toc(t0)),' seconds.'] ) ;

%%

%Update figure - i.e. change is the transparency or texture, not the
%surface itself
function update_figure
global d
figure(d.fig);
[d.az,d.el] = view;
delete(d.s)
d.s = plot_surface(d.x,d.y,d.z, d.texture, d.transparency, d.B, d.T, d.light_effect );
view(d.az,d.el);

%%

%Plot surface
function s = plot_surface( x,y,z,texture,transparency, B, T, light_effect )

%Create surface object s
s = surf( x,y,z);
shading interp;
interp_colormap(max(size(x)));
view([110,36]);
axis vis3d
axis off
zoom(1.5);

%Set colours of surface points to match those of bitmap, i.e.
%texture-mapping the bitmap.
if texture ==1
    set( s, 'CDatamapping', 'Direct','CData', B );
end

%Set the transparency of the surface
if transparency == 1
    set( s, 'AlphaDataMapping','none','AlphaData', T,...
        'edgealpha','interp','facealpha','interp' );
end

%Add camlight
if light_effect == 1
    camlight;
end

%%

%Key press handler
function keypressfunc( fig,evnt )

%Allow read-write usage of global variable d
global d

%Get the string of the current key press
s = get( d.fig,'currentkey');

%Actions, depending on the key pressed...

if strcmp(s,'q') == 1
    %Quit
    close(d.fig)
    
elseif strcmp(s, 't') == 1
    %Toggle transparency
    if d.transparency==1
        d.transparency=0;
    else
        d.transparency=1;
    end
    update_figure
    
elseif strcmp(s, 'y') == 1
    %Load new transparency
    [d.T,d.transparency_fname] = get_image( d.width_pix, d.height_pix, [] );
    d.transparency=1;
    update_figure
    
elseif strcmp(s, 'b') == 1
    %Toggle textured surface
    if d.texture==1
        d.texture=0;
    else
        d.texture=1;
    end
    update_figure
    
elseif strcmp(s, 'n') == 1
    %Load new textured surface
    [d.B,d.texture_fname] = get_image( d.width_pix, d.height_pix, [] );
    d.texture = 1;
    update_figure
    
elseif strcmp(s, 'l') == 1
    %Toggle camlight
    if d.light_effect == 1
        d.light_effect = 0;
    else
        d.light_effect = 1;
    end
    update_figure;
    
elseif strcmp(s, 'h') == 1
    
    %Toggle high resolution
    if d.high_res == 1
        d.high_res = 0;
    else
        d.high_res = 1;
    end
    
    %Replace-surface with modified resolution (high-res or original)
    disp(' ')
    disp(' Toggling high resolution image, this may take a while....')
    t0 = tic;
    
    if d.high_res == 1
        %Replace image dimensions with high resolution version
        d.width_pix = d.high_res_width_pix;
        d.height_pix = d.high_res_height_pix;
    else
        %Replace image dimensions with original resolution version
        d.width_pix = d.orig_width_pix;
        d.height_pix = d.orig_height_pix;
    end
    
    %Reload texture and transpareny bitmaps
    [d.B,d.texture_fname] = get_image( d.width_pix, d.height_pix, d.texture_fname );
    [d.T,d.transparency_fname] = get_image( d.width_pix, d.height_pix, d.transparency_fname );
    
    %Re-generate surface at this new resolution
    [d.az,d.el] = view;
    generate_surface;
    
    %Set previous view
    view(d.az,d.el);
    
    %Display completion message to command window
    disp(' ')
    disp([ ' Surface re-generated in ',num2str(toc(t0)),' seconds.'] ) ;
    
elseif strcmp(s, 'p') == 1
    %Print figure
    print(d.fig,['spherium image ',strrep(datestr(now),':','-')],'-djpeg','-r600');
    
elseif strcmp(s, 'v') == 1
    %Display current view to command window
    disp(' ')
    [d.az,d.el] = view;
    disp([' Current [az,el] view is:   [',num2str(d.az),',',num2str(d.el),']' ] );
    
elseif strcmp(s, 's') == 1
    
    %If transparency applied, drill holes in the surface
    if d.transparency==1;
        
        %Set max value of transparency for holes to be 'drilled'
        Tmax = 0.1;
        
        %Combine transparency into a single matrix
        %Make sure it is only one 'page' i.e. greyscale. If not mean over the
        %other colours.
        [r,c,p] = size(d.T);
        if p~=1
            T = zeros(r,c);
            T = (1/3) * ( d.T(:,:,1)+d.T(:,:,2)+d.T(:,:,3) );
        else
            T=d.T;
        end
        
        %Store original x,y,z coordinates
        x = d.x;
        y = d.y;
        z = d.z;
 
        %Drill holes via setting x,y,z coordinates to be NaN
        d.x(T<Tmax) = NaN;
        d.y(T<Tmax) = NaN;
        d.z(T<Tmax) = NaN;
        clear T
        update_figure;
    end
    
    %Export .stl file of current surface
    stlwrite( ['spherium image ',strrep(datestr(now),':','-'),'.stl'],surf2patch(d.s,'triangles'));
    
    %Restore figure, if transparency has caused holes to be drilled
    if d.transparency==1;
        d.x = x;
        d.y = y;
        d.z = z;
        clear x y z
        update_figure;
    end
end

%%


function stlwrite(filename, varargin)
%STLWRITE   Write STL file from patch or surface data.
%
%   STLWRITE(FILE, FV) writes a stereolithography (STL) file to FILE for a
%   triangulated patch defined by FV (a structure with fields 'vertices'
%   and 'faces').
%
%   STLWRITE(FILE, FACES, VERTICES) takes faces and vertices separately,
%   rather than in an FV struct
%
%   STLWRITE(FILE, X, Y, Z) creates an STL file from surface data in X, Y,
%   and Z. STLWRITE triangulates this gridded data into a triangulated
%   surface using triangulation options specified below. X, Y and Z can be
%   two-dimensional arrays with the same size. If X and Y are vectors with
%   length equal to SIZE(Z,2) and SIZE(Z,1), respectively, they are passed
%   through MESHGRID to create gridded data. If X or Y are scalar values,
%   they are used to specify the X and Y spacing between grid points.
%
%   STLWRITE(...,'PropertyName',VALUE,'PropertyName',VALUE,...) writes an
%   STL file using the following property values:
%
%   MODE          - File is written using 'binary' (default) or 'ascii'.
%
%   TITLE         - Header text (max 80 chars) written to the STL file.
%
%   TRIANGULATION - When used with gridded data, TRIANGULATION is either:
%                       'delaunay'  - (default) Delaunay triangulation of X, Y
%                       'f'         - Forward slash division of grid quads
%                       'b'         - Back slash division of quadrilaterals
%                       'x'         - Cross division of quadrilaterals
%                   Note that 'f', 'b', or 't' triangulations now use an
%                   inbuilt version of FEX entry 28327, "mesh2tri".
%
%   FACECOLOR     - Single colour (1-by-3) or one-colour-per-face (N-by-3) 
%                   vector of RGB colours, for face/vertex input. RGB range
%                   is 5 bits (0:31), stored in VisCAM/SolidView format
%                   (http://en.wikipedia.org/wiki/STL_(file_format)#Color_in_binary_STL)
%
%   Example 1:
%       % Write binary STL from face/vertex data
%       tmpvol = zeros(20,20,20);       % Empty voxel volume
%       tmpvol(8:12,8:12,5:15) = 1;     % Turn some voxels on
%       fv = isosurface(tmpvol, 0.99);  % Create the patch object
%       stlwrite('test.stl',fv)         % Save to binary .stl
%
%   Example 2:
%       % Write ascii STL from gridded data
%       [X,Y] = deal(1:40);             % Create grid reference
%       Z = peaks(40);                  % Create grid height
%       stlwrite('test.stl',X,Y,Z,'mode','ascii')

%   Original idea adapted from surf2stl by Bill McDonald. Huge speed
%   improvements implemented by Oliver Woodford. Non-Delaunay triangulation
%   of quadrilateral surface courtesy of Kevin Moerman. FaceColor
%   implementation by Grant Lohsen.
%
%   Author: Sven Holcombe, 11-24-11


% Check valid filename path
path = fileparts(filename);
if ~isempty(path) && ~exist(path,'dir')
    error('Directory "%s" does not exist.',path);
end

% Get faces, vertices, and user-defined options for writing
[faces, vertices, options] = parseInputs(varargin{:});
asciiMode = strcmp( options.mode ,'ascii');

% Create the facets
facets = single(vertices');
facets = reshape(facets(:,faces'), 3, 3, []);

% Compute their normals
V1 = squeeze(facets(:,2,:) - facets(:,1,:));
V2 = squeeze(facets(:,3,:) - facets(:,1,:));
normals = V1([2 3 1],:) .* V2([3 1 2],:) - V2([2 3 1],:) .* V1([3 1 2],:);
clear V1 V2
normals = bsxfun(@times, normals, 1 ./ sqrt(sum(normals .* normals, 1)));
facets = cat(2, reshape(normals, 3, 1, []), facets);
clear normals

% Open the file for writing
permissions = {'w','wb+'};
fid = fopen(filename, permissions{asciiMode+1});
if (fid == -1)
    error('stlwrite:cannotWriteFile', 'Unable to write to %s', filename);
end

% Write the file contents
if asciiMode
    % Write HEADER
    fprintf(fid,'solid %s\r\n',options.title);
    % Write DATA
    fprintf(fid,[...
        'facet normal %.7E %.7E %.7E\r\n' ...
        'outer loop\r\n' ...
        'vertex %.7E %.7E %.7E\r\n' ...
        'vertex %.7E %.7E %.7E\r\n' ...
        'vertex %.7E %.7E %.7E\r\n' ...
        'endloop\r\n' ...
        'endfacet\r\n'], facets);
    % Write FOOTER
    fprintf(fid,'endsolid %s\r\n',options.title);
    
else % BINARY
    % Write HEADER
    fprintf(fid, '%-80s', options.title);             % Title
    fwrite(fid, size(facets, 3), 'uint32');           % Number of facets
    % Write DATA
    % Add one uint16(0) to the end of each facet using a typecasting trick
    facets = reshape(typecast(facets(:), 'uint16'), 12*2, []);
    % Set the last bit to 0 (default) or supplied RGB
    facets(end+1,:) = options.facecolor;
    fwrite(fid, facets, 'uint16');
end

% Close the file
fclose(fid);
fprintf('Wrote %d facets\n',size(facets, 3));


%% Input handling subfunctions
function [faces, vertices, options] = parseInputs(varargin)
% Determine input type
if isstruct(varargin{1}) % stlwrite('file', FVstruct, ...)
    if ~all(isfield(varargin{1},{'vertices','faces'}))
        error( 'Variable p must be a faces/vertices structure' );
    end
    faces = varargin{1}.faces;
    vertices = varargin{1}.vertices;
    options = parseOptions(varargin{2:end});
    
elseif isnumeric(varargin{1})
    firstNumInput = cellfun(@isnumeric,varargin);
    firstNumInput(find(~firstNumInput,1):end) = 0; % Only consider numerical input PRIOR to the first non-numeric
    numericInputCnt = nnz(firstNumInput);
    
    options = parseOptions(varargin{numericInputCnt+1:end});
    switch numericInputCnt
        case 3 % stlwrite('file', X, Y, Z, ...)
            % Extract the matrix Z
            Z = varargin{3};
            
            % Convert scalar XY to vectors
            ZsizeXY = fliplr(size(Z));
            for i = 1:2
                if isscalar(varargin{i})
                    varargin{i} = (0:ZsizeXY(i)-1) * varargin{i};
                end                    
            end
            
            % Extract X and Y
            if isequal(size(Z), size(varargin{1}), size(varargin{2}))
                % X,Y,Z were all provided as matrices
                [X,Y] = varargin{1:2};
            elseif numel(varargin{1})==ZsizeXY(1) && numel(varargin{2})==ZsizeXY(2)
                % Convert vector XY to meshgrid
                [X,Y] = meshgrid(varargin{1}, varargin{2});
            else
                error('stlwrite:badinput', 'Unable to resolve X and Y variables');
            end
            
            % Convert to faces/vertices
            if strcmp(options.triangulation,'delaunay')
                faces = delaunay(X,Y);
                vertices = [X(:) Y(:) Z(:)];
            else
                if ~exist('mesh2tri','file')
                    error('stlwrite:missing', '"mesh2tri" is required to convert X,Y,Z matrices to STL. It can be downloaded from:\n%s\n',...
                        'http://www.mathworks.com/matlabcentral/fileexchange/28327')
                end
                [faces, vertices] = mesh2tri(X, Y, Z, options.triangulation);
            end
            
        case 2 % stlwrite('file', FACES, VERTICES, ...)
            faces = varargin{1};
            vertices = varargin{2};
            
        otherwise
            error('stlwrite:badinput', 'Unable to resolve input types.');
    end
end

if ~isempty(options.facecolor) % Handle colour preparation
    facecolor = uint16(options.facecolor);
    %Set the Valid Color bit (bit 15)
    c0 = bitshift(ones(size(faces,1),1,'uint16'),15);
    %Red color (10:15), Blue color (5:9), Green color (0:4)
    c0 = bitor(bitshift(bitand(2^6-1, facecolor(:,1)),10),c0);
    c0 = bitor(bitshift(bitand(2^11-1, facecolor(:,2)),5),c0);
    c0 = bitor(bitand(2^6-1, facecolor(:,3)),c0);
    options.facecolor = c0;    
else
    options.facecolor = 0;
end

function options = parseOptions(varargin)
IP = inputParser;
IP.addParamValue('mode', 'binary', @ischar)
IP.addParamValue('title', sprintf('Created by stlwrite.m %s',datestr(now)), @ischar);
IP.addParamValue('triangulation', 'delaunay', @ischar);
IP.addParamValue('facecolor',[], @isnumeric)
IP.parse(varargin{:});
options = IP.Results;

function [F,V]=mesh2tri(X,Y,Z,tri_type)
% function [F,V]=mesh2tri(X,Y,Z,tri_type)
% 
% Available from http://www.mathworks.com/matlabcentral/fileexchange/28327
% Included here for convenience. Many thanks to Kevin Mattheus Moerman
% kevinmoerman@hotmail.com
% 15/07/2010
%------------------------------------------------------------------------

[J,I]=meshgrid(1:1:size(X,2)-1,1:1:size(X,1)-1);

switch tri_type
    case 'f'%Forward slash
        TRI_I=[I(:),I(:)+1,I(:)+1;  I(:),I(:),I(:)+1];
        TRI_J=[J(:),J(:)+1,J(:);   J(:),J(:)+1,J(:)+1];
        F = sub2ind(size(X),TRI_I,TRI_J);
    case 'b'%Back slash
        TRI_I=[I(:),I(:)+1,I(:);  I(:)+1,I(:)+1,I(:)];
        TRI_J=[J(:)+1,J(:),J(:);   J(:)+1,J(:),J(:)+1];
        F = sub2ind(size(X),TRI_I,TRI_J);
    case 'x'%Cross
        TRI_I=[I(:)+1,I(:);  I(:)+1,I(:)+1;  I(:),I(:)+1;    I(:),I(:)];
        TRI_J=[J(:),J(:);    J(:)+1,J(:);    J(:)+1,J(:)+1;  J(:),J(:)+1];
        IND=((numel(X)+1):numel(X)+prod(size(X)-1))';
        F = sub2ind(size(X),TRI_I,TRI_J);
        F(:,3)=repmat(IND,[4,1]);
        Fe_I=[I(:),I(:)+1,I(:)+1,I(:)]; Fe_J=[J(:),J(:),J(:)+1,J(:)+1];
        Fe = sub2ind(size(X),Fe_I,Fe_J);
        Xe=mean(X(Fe),2); Ye=mean(Y(Fe),2);  Ze=mean(Z(Fe),2);
        X=[X(:);Xe(:)]; Y=[Y(:);Ye(:)]; Z=[Z(:);Ze(:)];
end

V=[X(:),Y(:),Z(:)];

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
%
% LAST UPDATED by Andy French 14 September 2008

function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%%

%get_image
% Open a bitmap image file and return a colour matrix (double precision, in
% range [0,1]) that can be used for surface texture mapping. The bitmap
% will be interpolated based upon the dimensions provided. If one or more
% of the dimensions is [] then the size of the colour matrix will be
% determined by the size of the bitmap.
%
% Syntax: I = get_image( width_pix, height_pix, fname )
%
% width_pix     Width of I in pixels.
% height_pix    Height of I in pixels.
% fname         I [] then a file browser box will open requesting a file.
%               If a genuine pathname to a file, this file will be loaded.
%
% I             R,G,B Colour matrix. Elements are floating point numbers from
%               range [0,1]. Dimensions are height_pix x width_pix x 3
%
% LAST UPDATED by Andy French 19/1/2014

function [I,fname] = get_image( width_pix, height_pix, fname )

% Choose bitmap file
if isempty(fname)
    [filename, pathname, filterindex] = uigetfile( {'*.jpg';'*.png';'*.bmp';'*.gif'},...
        'Choose bitmap image file') ;
    fname = [pathname,filename];
end

% Load bitmap file into array I. Convert it into a matrix of numbers
% ranging from 0 to 1.
try
    %Try and import image file
    [I,cmap] = imread(fname);
    
    %Convert image to double precision on [0,1]
    I = double(I)/255 ;
    I = flipdim(I,1);
    
    %Get dimensions of I
    [heightI,widthI,num_cols]  = size(I);
    
    %Take into account grayscale images, which are a single colour plane.
    %If the image is grayscale, repeat it for R,G and B planes.
    if num_cols==1
        II = zeros([heightI,widthI,3]);
        II(:,:,1) = I;
        II(:,:,2) = I;
        II(:,:,3) = I;
        I = II;
        clear II;
    end
    
    %Interpolate image if required
    if ( ~isempty( width_pix) ) && ( ~isempty( height_pix ) )
        
        %Define index vectors for height and width of 'initial_image'
        w = 1:widthI;
        h = 1:heightI;
        
        %Define index vectors for height and width of output image
        W = linspace(1,widthI,width_pix );
        H = linspace(1,heightI,height_pix );
        
        %Meshgrid these
        [h,w] = meshgrid(h,w);
        [H,W] = meshgrid(H,W);
        
        %Create R,G,B matricies from image matrix I.
        R = ones(heightI,widthI);
        G = ones(heightI,widthI);
        B = ones(heightI,widthI);
        R(:,:) = I(:,:,1);
        G(:,:) = I(:,:,2);
        B(:,:) = I(:,:,3);
        
        %Interpolate these colour matricies
        R = interp2(h,w,R.',H,W);
        G = interp2(h,w,G.',H,W);
        B = interp2(h,w,B.',H,W);
        clear h H w W
        
        %Initialise output array
        dim = size(R.');
        II = zeros(dim(1),dim(2),3);
        II(:,:,1) = R.';
        II(:,:,2) = G.';
        II(:,:,3) = B.';
        clear R G B
        
        %Assign output
        I = II;
        clear II;
    end
    
catch exception
    
    %Display file loading error to comand window
    disp(' ')
    %disp( exception )
    
    %If file doesn't exist or doesn't load, set random colours
    if ( isempty( width_pix) ) || ( isempty( height_pix ) )
        %Default image dimensions if no options set
        width_pix = 200;
        height_pix = 100;
    end
    I = rand( width_pix, height_pix, 3 );
end

%End of code