%standing_waves
% GUI to assist teaching and visualization of standing waves, using a
% spatial 'photograph' or snapshot, and three time variations at different
% spatial locations.
%
% LAST UPDATED by Andy French November 2023

function varargout = standing_waves(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @standing_waves_OpeningFcn, ...
    'gui_OutputFcn',  @standing_waves_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% standing_waves opening function. This executes just before forces is made visible.
function standing_waves_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for forces
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;
d.legend = [];

%Center figure
figcentre( d.h.FIGUREstanding_waves );

%Set defaults
standing_waves_defaults

%Update forces
standing_waves_update;

%%

% Default parameters
function standing_waves_defaults
global d

%Font size for graphs
d.fsize = 18;

%Time where wave starts (in fraction of a period)
d.T_frac = 0;

%Wave maximum amplitude
d.A = 1;

%Nodes at both ends? (Otherwise antinode at left end, node at right end)
d.node_node = 1;
set( d.h.CHECKnodenode,'value',d.node_node );

%Length of standing wave /m
d.L = 1.0;

%Wave speed (m/s)
d.c = 340;

%Harmoinic number
d.N = 2;

%Set up harmonic number pop up menu
set( d.h.MENUharmonicnumberN,'string',...
    {1,2,3,4,5,6,7,8,9,10},'value',d.N );

%Define x values for three time varying traces
d.x1 = 0.2*d.L;
d.x2 = 0.4*d.L;
d.x3 = 0.8*d.L;

%Update wavespeed and length eidt boxes
set(d.h.EDITwavespeed,'string',num2str(d.c));
set(d.h.EDITlength,'string',num2str(d.L));

%Update time variation spatial points edit boxes
%Update wavespeed and length eidt boxes
set(d.h.EDITx1,'string',num2str(d.x1));
set(d.h.EDITx2,'string',num2str(d.x2));
set(d.h.EDITx3,'string',num2str(d.x3));

%%

%% Update GUI
function standing_waves_update
global d

%Calculate frequencies /Hz
if d.node_node==1
    %L is a whole number of half wavelengths
    d.f = d.N*d.c/(2*d.L);
else
    %L is an odd number of quarter wavelengths i.e. antinode at left and
    %and node at right end
    d.f = (2*d.N-1)*d.c/(4*d.L);
end

%Calculate wave period /s and wavenumber k (m^-1) and angular frequency w
%(rads^-1) and wavelength (m)
d.T = 1/d.f; d.lamda =d.c/d.f;  d.k = 2*pi/d.lamda; d.w = 2*pi*d.f;

%Get time of wave /s and update text box
d.t = d.T_frac*d.T;
set(d.h.TEXTtime,'string',['t = ',num2str( d.t ,4 ),' s'] );
set(d.h.TEXTtoverT,'string',['t / T = ',num2str( d.T_frac ,4 ) ] );

%Plot standing wave envelope
axes(d.h.AXESstandingwave); cla; hold on;
x = linspace(0,d.L,1000); tt = linspace(0,d.T,21);
[tt,xx] = meshgrid(tt,x);
if d.node_node==1
    yy = d.A*sin( d.k*xx ).*cos(d.w*tt);
else
    yy = d.A*cos( d.k*xx ).*cos(d.w*tt);
end
plot(xx,yy); set(gca,'fontsize',d.fsize );
xlabel('x/m'); grid on; box on; 
title(['f=',num2str(d.f,4),'Hz, \lambda=',num2str(d.lamda,4),'m, T=',num2str(d.T,4),'s',...
    ', k=',num2str(d.k,4),'m^{-1}, \omega=',num2str(d.w,4),'rads^{-1}']);

%Define and plot standing wave y(x,t)
if d.node_node==1
    y = d.A*sin( d.k*x )*cos(d.w*d.t);
else
    y = d.A*cos( d.k*x )*cos(d.w*d.t);
end
plot(x,y,'linewidth',3,'color','g');

%Plot vertical lines for time varying traces
plot( d.x1*[1,1], d.A*[-1,1],'r','linewidth',2 );
plot( d.x2*[1,1], d.A*[-1,1],'k','linewidth',2 );
plot( d.x3*[1,1], d.A*[-1,1],'b','linewidth',2 );

%Update oscilloscope time-variation traces
tt = linspace( d.t, d.t + d.T*5, 1000 );
if d.node_node==1
    y1 = d.A*sin( d.k*d.x1 )*cos(d.w*tt);
    y2 = d.A*sin( d.k*d.x2 )*cos(d.w*tt);
    y3 = d.A*sin( d.k*d.x3 )*cos(d.w*tt);
else
    y1 = d.A*cos( d.k*d.x1 )*cos(d.w*tt);
    y2 = d.A*sin( d.k*d.x2 )*cos(d.w*tt);
    y3 = d.A*sin( d.k*d.x3 )*cos(d.w*tt);
end
axes(d.h.AXES1); cla; hold on;
plot(tt,y1,'r'); xlabel('t /s'); grid on; box on; ylim([-d.A,d.A]);
axes(d.h.AXES2); cla; hold on;
plot(tt,y2,'k'); xlabel('t /s'); grid on; box on; ylim([-d.A,d.A]);
axes(d.h.AXES3); cla; hold on;
plot(tt,y3,'b'); xlabel('t /s'); grid on; box on; ylim([-d.A,d.A]);

%%

%% Output function. Outputs from this function are returned to the command line.
function varargout = standing_waves_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Reduce time pushbutton
function PUSHreducetime_Callback(hObject, eventdata, handles)
global d
d.T_frac = d.T_frac - 0.02;
standing_waves_update

%%

%% Increase time pushbutton
function PUSHincreasetime_Callback(hObject, eventdata, handles)
global d
d.T_frac = d.T_frac + 0.02;
standing_waves_update

%%

%% AXES1 x1 edit box
function EDITx1_Callback(hObject, eventdata, handles)
global d
d.x1 = str2num( get( d.h.EDITx1,'string') );
if d.x1<0; d.x1 = 0; set( d.h.EDITx1,'string',num2str(d.x1)); end
if d.x1>d.L; d.x1 = d.L; set( d.h.EDITx1,'string',num2str(d.x1)); end
standing_waves_update

%%

%% Increase x1 pushbutton
function PUSHx1plus_Callback(hObject, eventdata, handles)
global d
d.x1 = d.x1 + 0.01*d.L;
if d.x1 > d.L; d.x1 = d.L; end
set( d.h.EDITx1,'string',num2str(d.x1,4) );
standing_waves_update

%%

%% Decrease x1 pushbutton
function PUSHx1minus_Callback(hObject, eventdata, handles)
global d
d.x1 = d.x1 - 0.01*d.L;
if d.x1 < 0; d.x1 = 0; end
set( d.h.EDITx1,'string',num2str(d.x1,4) );
standing_waves_update

%%

%% AXES2 x2 edit box
function EDITx2_Callback(hObject, eventdata, handles)
global d
d.x2 = str2num( get( d.h.EDITx1,'string') );
if d.x2<0; d.x2 = 0; set( d.h.EDITx2,'string',num2str(d.x2)); end
if d.x2>d.L; d.x2 = d.L; set( d.h.EDITx2,'string',num2str(d.x2)); end
standing_waves_update

%%

%% Increase x2 pushbutton
function PUSHx2plus_Callback(hObject, eventdata, handles)
global d
d.x2 = d.x2 + 0.01*d.L;
if d.x2 > d.L; d.x2 = d.L; end
set( d.h.EDITx2,'string',num2str(d.x2,4) );
standing_waves_update

%%

%% Decrease x1 pushbutton
function PUSHx2minus_Callback(hObject, eventdata, handles)
global d
d.x2 = d.x2 - 0.01*d.L;
if d.x2 < 0; d.x2 = 0; end
set( d.h.EDITx2,'string',num2str(d.x2,4) );
standing_waves_update

%%

%% AXES3 x3 edit box
function EDITx3_Callback(hObject, eventdata, handles)
global d
d.x3 = str2num( get( d.h.EDITx3,'string') );
if d.x3<0; d.x3 = 0; set( d.h.EDITx3,'string',num2str(d.x3)); end
if d.x3>d.L; d.x3 = d.L; set( d.h.EDITx3,'string',num2str(d.x3)); end
standing_waves_update

%%

%% Increase x3 pushbutton
function PUSHx3plus_Callback(hObject, eventdata, handles)
global d
d.x3 = d.x3 + 0.01*d.L;
if d.x1 > d.L; d.x1 = d.L; end
set( d.h.EDITx3,'string',num2str(d.x3,4) );
standing_waves_update

%%

%% Decrease x3 pushbutton
function PUSHx3minus_Callback(hObject, eventdata, handles)
global d
d.x3 = d.x3 - 0.01*d.L;
if d.x3 < 0; d.x3 = 0; end
set( d.h.EDITx1,'string',num2str(d.x3,4) );
standing_waves_update

%%

%% Harmonic number pop up menu
function MENUharmonicnumberN_Callback(hObject, eventdata, handles)
global d
d.N = get( d.h.MENUharmonicnumberN,'value' );
standing_waves_update

%%

%% Wave speed edit box
function EDITwavespeed_Callback(hObject, eventdata, handles)
global d
d.c = str2num( get(d.h.EDITwavespeed,'string') );
standing_waves_update

%%

%% Length edit box
function EDITlength_Callback(hObject, eventdata, handles)
global d
d.L = str2num( get(d.h.EDITlength,'string') );
standing_waves_update

%%

%% Nodes at both ends checkbox
function CHECKnodenode_Callback(hObject, eventdata, handles)
global d
d.node_node = get( d.h.CHECKnodenode,'value' );
standing_waves_update

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code






