%solve_sudoku

function [x,P] = solve_sudoku(x,na)

%%%% Initializations %%%%

%Initialise arrays used in the sudoku solver
P = cell(na,na);

%Initialise solved parameter
solved = 0;

max_iter = 20;
iter=0;

%

%%%% Solver %%%%

%Try deductive solver first. When this cannot proceed any further, try
%hypothesis based solver, which will step through the smallest set of
%non-scalar elements of possibilities matrix P and seeing if the deductive
%solver eiether (i) solves the sudoku or (ii) returns a bad result
while (solved==0) && (iter<max_iter)

    %Try deductive solver. Unless sudoku has been badly formed this will
    %not return a bad==1 result, only a solved==1 or solved==0 result.
    [x,P,solved,bad] = deductive_solver(x,P,na);

    %If sudoku is bad at this stage, return error message
    if bad==1;
        error('Poorly formed sudoku. Check dimensions and possibility of duplication of elements')
    end

    %If sudoku has not been solved during the deductive stage, store the
    %current sudoku and then make a hypothesis. Apply deduction (and
    %possibly further hypothesis) until (i) the sudoku is solved, or (ii) a
    %bad sudoku is returned. If the latter is the case then return to the
    %stored sudoku and try the next guess.
    [x,P,solved,bad] = hypo_solver(x,P,na);
    
    %Increment iterations
    iter = iter + max_iter;
end

%%

%Deductive sudoku solver
function [x,P,solved,bad] = deductive_solver(x,P,na)

%Assume sudoku is not initially solved
solved = 0;

%Check whether sudoku is bad. If so, exit function
bad=check_sudoku(x,na);
if bad==1;
    return;
end

%Check whether sudoku x is solved. If so, exit function. Solved implies no
%remaining zero elements in the sudoku x
if sum(sum( x==0 ))==0
    solved = 1;
    return
end

%Attempt to solve suduko via deduction until subsequent iterations cease to
%yield a change in the sudoku. Initialise dz which is the change in number
%of undefined elements (zeros) in the sudoku x between iterations
dz = NaN;
z = numel(x==0);
while dz~=0
    %Update possibilities array P based on current sudoku x
    [P,bad] = compute_possibilities( na,x,P );

    %Check whether sudoku is bad. If so, exit function
    bad=check_sudoku(x,na);
    if bad==1;
        return;
    end

    %Update x based upon any deduction made following update of P. i.e.
    %elements which are now scalars, not 'vectors of possibility'
    x = deductions( na, x, P );

    %Check whether sudoku is bad. If so, exit function
    bad=check_sudoku(x,na);
    if bad==1;
        return;
    end

    %Compute change in number of zeros, and update store of number of zeros
    dz = length(find(x==0)) - z;
    z = length(find(x==0));
end

%%

%Hypothesis based sudoku solver
function [x,P,solved,bad] = hypo_solver(x,P,na)

%Update stored x
x_old = x;
P_old = P;

%Determine multi valued elements of P
zm = [];
zi = [];
for z=1:numel(P_old)
    if length( P_old{z} ) > 1
        zm = [ zm, length(P_old{z}) ];
        zi = [ zi, z];
    end
end

%Sort these and choose smallest
[zm,zz] = sort(zm);
z = zi(zz(1));

%Step through elements of P{z} and see if a logical inconsistency
%arises
good_guesses = ones( 1,length(P_old{z}) );
for q=1:length(P_old{z})
    [n,m] = ind2sub([na,na],z);
    x(n,m) = P_old{z}(q);

    %Try deductive solver. Unless sudoku has been badly formed this will
    %not return a bad==1 result, only a solved==1 or solved==0 result.
    [x,P,solved,bad] = deductive_solver(x,P,na);

    %Check whether sudoku is bad. If so, skip to next loop, and revert the
    %sudoku and possibilities array to the stored versions
    bad=check_sudoku(x,na);
    if bad==1;
        x = x_old;
        P = P_old;
        good_guesses(q) = 0;
        continue;
    end

    %Check whether sudoku x is solved. If so, exit function. Solved implies no
    %remaining zero elements in the sudoku x
    if sum(sum( x==0 ))==0
        solved = 1;
        return
    end

    % This causes an infinite loop?
%     %If sudoku has not yet been solved by deduction and is not yet bad, try
%     %another round of hypothesis solver
%     [x,P,solved,bad] = hypo_solver(x,P,na);
%     if bad==1;
%         x = x_old;
%         P = P_old;
%         good_guesses(q) = 0;
%         continue;
%     end
end

%Check if any of the guesses were good. The only possibilities are one or
%none.
if sum(good_guesses==1)==1
    %If we haven't already solved the sudoku by now, deduce that the guess g
    %corresponding to good_guesses(g)==1 was correct, (i.e. because but further
    %work is required to solve the sudoku.
    [n,m] = ind2sub([na,na],z);
    x(n,m) = P_old{z}( find(good_guesses==1) );
end


%%

%Compute 'possibilities' cell array P
function [P,bad] = compute_possibilities( na,x,P )

%Initialise 'bad' flag. bad==1 indicates a logical inconsistency, i.e.
%one element of P is [], or the sudoku is badly formed (e.g. with
%duplicates in the H,V and S sets
bad = 0;

%Compute Horizontal, Vertical and Square sets
a = 1:na;
H = cell(1,na);
V = cell(1,na);
S = cell(1,na);
for n=1:na
    H{n} = setdiff( a, x(n,:) );
    V{n} = setdiff( a, x(:,n) );
    S{n} = setdiff( a, square_set( x, n ) );
end

%Compute 'possibilities' cell array P
for n=1:na
    for m=1:na

        %Element (n,m) is an unknown number in sudoku
        if x(n,m)==0

            %Determine square set index
            ns = square_set_index( na, n,m );

            %Determine sudoku indices of elements of square set
            %containing sudoku cell (n,m)
            [n_square,m_square] = sudoku_indices_from_S(ns,na);

            %Compute possibilities for sudoku number (n,m)
            P{n,m} = intersect( S{ns},...
                intersect( H{n}, V{m} ) );
        else
            %Element (n,m) is a defined number in sudoku.
            %There is only one possibility for this element!
            P{n,m} = x(n,m) ;
        end

        %Check if P{n,m} is empty. If so, update bad flag
        if isempty( P{n,m} ) == 1
            bad = 1;
        end
    end
end

%%

%Check integrity of suduko
function bad=check_sudoku(x,na)

%Assume sudoku is intially OK
bad = 0;

%Check that sudoku is of the correct dimensions
dim = size(x);
if (dim(1)~=na) || (dim(2)~=na)
    bad=1;
    return
end

%Check for duplicate non-zero elements in H,V and S sets
for n=1:na
    H = x(n,:); H(H==0) = [];
    V = x(:,n);  V(V==0) = [];
    S = square_set( x, n );  S(S==0) = [];
    if (areduplicates(H)==1) || (areduplicates(V)==1) || (areduplicates(S)==1)
        bad = 1;
        break
    end
end

%%

%Check sudoku x for new deductions in 'possibilities array' P. If a
%deduction is made, update sudoku
function x = deductions( na, x, P )
for z=1:numel(P)
    [n,m] = ind2sub([na,na],z);
    if length( P{z} )==1
        x(n,m) = P{n,m};
    end
end

%%

% Find nth square set (returned as a vector) from sudoku x
function s = square_set( x, n )

%Define size of square set. i.e. a square set has N^2 elements.
N = sqrt(sqrt(numel(x)));

%Find (row,column) indices of chosen square set, noting the sudoku is a
%set of N^2 square sets
[r,c] = ind2sub( [N,N], n );

%Sample sudoku x to yield output s, a row vector
s = x( 1 + (r-1)*N : r*N , 1 + (c-1)*N : c*N );
s = reshape( s, [1,N*N] );

%%

% Return square-set index from sudoku indices (p,q). p is the row number
% and q is the column number. na is the length of the sudoku alphabet and
% n_s is the square set number (which will be from the set 1:na )
function n_s = square_set_index( na, p,q )
s = sqrt(na);
n_s = sub2ind( [s,s], ceil( p/s ), ceil( q/s ) );

%%

%Return sudoku indices p,q of elemnts of nth square set, for a sudoku with
%an alphabet of na elements
function [p,q] = sudoku_indices_from_S(n,na)
N = sqrt(na);
[r,c] = ind2sub( [N,N], n );
p = 1 + (r-1)*N : r*N;
q = 1 + (c-1)*N : c*N;
[p,q]=meshgrid(p,q);
p = reshape(p,[1,na]);
q = reshape(q,[1,na]);

%%

%Binary function which indicates whether a vector x has duplicate elements
function y = areduplicates(x)
y = length(x) ~= length(unique(x));

%End of code