%solve_sudoku

function y = solve_sudoku(x)

%% Initializations %%

%Store initial sudoku
iter=1;
y(1).x_init = x;

%Define sudoku basic alphabet
na = sqrt(numel(x));
a = 1:na;

%Initialise Horizontal, Vertical and Square sets
H = cell(1,na);
V = cell(1,na);
S = cell(1,na);
for n=1:na
    H{n} = setdiff( a, x(n,:) );
    V{n} = setdiff( a, x(:,n) );
    S{n} = setdiff( a, square_set( x, n ) );
end

%Initialise 'possibilities' cell array P
P = cell(na,na);

%Initialise 'solved' binary flag. When this is unity, sudoku is solved!
solved = 0;

%Define max number of iterations
iter_max = 10;

%

%% Solve sudoku via an iterative process %%

while (solved == 0) && (iter <= iter_max)

    %Compute 'possibilities' cell array P
    for n=1:na
        for m=1:na

            %Unknown number in sudoku
            if x(n,m)==0

                %Determine square set index
                ns = square_set_index( na, n,m );

                %Determine sudoku indices of elements of square set
                %containing sudoku cell (n,m)
                [n_square,m_square] = sudoku_indices_from_S(ns,na);

                %Compute possibilities for sudoku number (n,m)
                P{n,m} = intersect( S{ns},...
                    intersect( H{n}, V{m} ) );

                %If a deduction is made (i.e. P{n,m} is a unique number),
                %update sudoku x and remove the deduced number from H, V and S
                %sets. Also remove instances of this deduction from all the
                %other sudoku elements in the intersecting H,V and S sets
                if length( P{n,m} ) == 1

                    
                end

                %Now step through remaining possibilities in associated H, V
                %sets with sudoku element (n,m) and remove the numbers from
                %the corresponding possibilities array P when the
                %possibility is ruled out because of a prior deduction in
                %an intersecting H,V or S set

                %Step through possibilities in H{n} and modify P{n,:}
                %accordingly
                for w=1:length(H{n})
                    for z=1:na
                        %Check possibilities in H{n} sudoku elements.
                        %Rule out those which are impossible, i.e.
                        %those which are already deduced in an S or V set
                        [n_square,m_square] = sudoku_indices_from_S( square_set_index( na, n,z ),na);
                        if ( x(n,z)==0 ) && ( ismember( H{n}(w), x(n_square,m_square) ) ||...
                                ismember( H{n}(w), x(:,z) )
                            P{n,z} = setdiff( P{n,z}, H{n}(w) );
                        end
                    end
                end

                %Step through possibilities in V{m} and modify P{n,:}
                %accordingly
                for w=1:length(V{m})
                    for z=1:na
                        %Check possibilities in V{m} sudoku elements.
                        %Rule out those which are impossible, i.e.
                        %those which are already deduced in an S or V set
                        [n_square,m_square] = sudoku_indices_from_S( square_set_index( na, z,m ),na);
                        if ( x(z,m)==0 ) && ( ismember( V{m}(w), x(n_square,m_square) ) ||...
                                ismember( V{m}(w), x(z,:) ) )
                            P{z,m} = setdiff( P{z,m}, V{m}(w) );
                        end
                    end
                end
            else
                %Defined number in sudoku. There is only one possibility
                %for this element!
                P{n,m} = x(n,m) ;
            end
        end
    end

    %Check number of zeros (i.e. undetermined numbers) in sudoku. When this
    %reaches 0, sudoku is solved!
    if isempty( find(x==0, 1) )
        solved = 1;
    end

    %Initialize output data structure
    y(iter).x = x;
    y(iter).H = H;
    y(iter).V = V;
    y(iter).S = S;
    y(iter).P = P;
    y(iter).solved = solved;

    %Update iteration counter
    iter = iter + 1;
end

%%

% Find nth square set (returned as a vector) from sudoku x
function s = square_set( x, n )

%Define size of square set. i.e. a square set has N^2 elements.
N = sqrt(sqrt(numel(x)));

%Find (row,column) indices of chosen square set, noting the sudoku is a
%set of N^2 square sets
[r,c] = ind2sub( [N,N], n );

%Sample sudoku x to yield output s, a row vector
s = x( 1 + (r-1)*N : r*N , 1 + (c-1)*N : c*N );
s = reshape( s, [1,N*N] );

%%

% Return square-set index from sudoku indices (p,q). p is the row number
% and q is the column number. na is the length of the sudoku alphabet and
% n_s is the square set number (which will be from the set 1:na )
function n_s = square_set_index( na, p,q )
s = sqrt(na);
n_s = sub2ind( [s,s], ceil( p/s ), ceil( q/s ) );

%%

%Return sudoku indices p,q of elemnts of nth square set, for a sudoku with
%an alphabet of na elements
function [p,q] = sudoku_indices_from_S(n,na)
N = sqrt(na);
[r,c] = ind2sub( [N,N], n );
p = 1 + (r-1)*N : r*N;
q = 1 + (c-1)*N : c*N;
[p,q]=meshgrid(p,q);
p = reshape(p,[1,na]);
q = reshape(q,[1,na]);

%%

%Check sudoku x for deductions in 'possibilities array' P. If a deduction
%is made.
function x = check_for_deductions(x,P,na)
for n=1:na
    for m=1:na
        if length( P{n,m} )==1
            x(n,m) = P{n,m};
        end
    end
end

%%

%Update sudoku as a result of deductions

%End of code