%SUDOKU
% Solver for Sudoku puzzels. A solved Sudoku puzzle is a 9 x 9 matrix of
% integers from the set 1 ...9 such that no column or row has repeated
% elements. Also all 3 x 3 boxes (that completely divide the 9 x 9 matrix)
% must not contain repeated elements!
%
% LAST UPDATED by Andy French. 3rd August 2005.
%
% Syntax:  S=sudoku(P)
% P is a 9 x 9 matrix of integers 0 ... 9
% 0 denotes an unknown integer from the set 1...9 to be found.
% S is the solution matrix, with unknowns determined.
%
% Example:
% P=[...
%     8,5,0,0,0,6,0,0,0;....
%     0,0,3,1,8,0,7,4,0;...
%     0,0,7,0,0,0,0,0,5;...
%     5,0,0,0,6,0,0,3,0;
%     0,8,0,0,0,0,0,7,0;...
%     0,6,0,0,4,0,0,0,8;...
%     9,0,0,0,0,0,2,0,0;,...
%     0,2,5,0,7,3,1,0,0;
%     0,0,0,2,0,0,0,6,3];
%
% S=sudoku(P)
%
% Example 2: (Tough! Daily Telegraph #54)
% P=[...
%     7,0,9,0,0,2,0,3,0;....
%     0,0,0,0,1,9,0,0,6;...
%     4,0,0,0,3,0,1,0,0;...
%     0,0,1,0,0,0,0,6,0;
%     0,0,8,2,0,7,9,0,0;...
%     0,4,0,0,0,0,5,0,0;...
%     0,0,6,0,8,0,0,0,9;,...
%     9,0,0,6,2,0,0,0,0;
%     0,5,0,9,0,0,6,0,4];
% S=sudoku(P)


function S=sudoku(P)

%Initialise cell array of possible integers. The size of the array matches
%P, but in general elements can be vectors of several integers.
for n=1:9
    for m=1:9
        Q{n,m}=P(n,m);
    end
end

%Divide P into 3 x 3 squares. For each square determine the 's' set of
%possible integers.
for n=1:3
    for m=1:3

        %Extract non zero elements of P from square (n,m) to form 's' set
        p=zeros(3,3);
        p(:,:)=P(3*(n-1)+1 : 3*(n-1)+3 , 3*(m-1)+1 : 3*(m-1)+3);
        s=[];
        for k=1:9
            if p(k)~=0
                s=[s,p(k)];
            end
        end
        s=setxor(s,1:9);

        %Form V and H sets corresponding to the non zero elements of P in
        %the vertical columns and horizontal rows that overlap square n,m

        for i=1:3
            %V
            v=zeros(1,9);
            v(:)=P(:,3*(m-1)+i);
            V{i}=[];
            for k=1:9
                if v(k)~=0
                    V{i}=[V{i},v(k)];
                end
            end
            V{i}=setxor(V{i},1:9);

            %H
            h=zeros(1,9);
            h(:)=P(3*(n-1)+i,:);
            H{i}=[];
            for k=1:9
                if h(k)~=0
                    H{i}=[H{i},h(k)];
                end
            end
            H{i}=setxor(H{i},1:9);
        end

        %For each of the zeros of p, determine possible integers that are
        %in the corresponding H, V & s set
        for x=1:3
            for y=1:3
                if p(x,y)==0
                    %H,V,s intersection
                    HVs=intersect(intersect(H{x},V{y}),s);

                    %If HVs is a single element, remove this from s
                    if length(HVs)==1
                        rm_index=find(s==HVs);
                        new_s=[];
                        for t=1:length(s)
                            if t~=rm_index
                                new_s=[new_s,s(t)];
                            end
                        end
                        s=new_s;
                    end

                    %Set vector element of Q
                    Q{3*(n-1)+x, 3*(m-1)+y}=HVs;
                end
            end
        end
    end
end

S=zeros(1,9);
for n=1:9
    for m=1:9
        if length(Q{n,m})==1
            S(n,m)=Q{n,m};
        end
    end
end
S

%While the number of indices stored within the Q cell array is >81
%Iterate around the following process:
%For each element of Q that has a length of >1, reset this as the
%intersection of itself with the 9 element vertical and horizontal vectors of
%indices representing those that are not already found. i.e. the rows and columns
%of the full 9 x 9 matrix that pass through the point in question.

%Initialise iteration counter iter
iter=0;
while (countq(Q) < 81) & (iter<=30)

    %Update iteration counter and count total number of integers in Q
    iter=iter+1
    oldcountq=countq(Q);

    %Determine elements of Q that have length >1 and store their content
    %and index number

    Qstore=[];
    Qn=[];
    Qm=[];
    k=1;
    for n=1:9
        for m=1:9
            if length(Q{n,m})>1
                Qstore{k}=Q{n,m};
                Qn(k)=n;
                Qm(k)=m;
                k=k+1;
            end
        end
    end
    clear k n m;

    %Step though the Qstore and set Q{n,m} to be the elements of
    %Qstore{k} (i.e. first, or second etc...) See if this causes countq(Q)
    %to decrease.

    for k=1:length(Qn)
        for y=1:length(Qstore{k})

            %Test to see whether yth element of Qstore k reduces the Q
            %count
            QQ=Q;
            QQ{Qn(k),Qm(k)}=Qstore{k}(y);

            %Remove this integer from the multi integer vector elements in
            %the rows and column that intersect the Qstore element
            for j=1:9
                mQdiff=setdiff(QQ{j,Qm(k)},Qstore{k}(y));
                nQdiff=setdiff(QQ{Qn(k),j},Qstore{k}(y));
                if (length(QQ{j,Qm(k)})>1) & (j~=Qn(k)) & (isempty(mQdiff)~=1)
                    QQ{j,Qm(k)}=mQdiff;
                end
                if (length(QQ{Qn(k),j})>1) & (j~=Qm(k)) & (isempty(nQdiff)~=1)
                    QQ{Qn(k),j}=nQdiff;
                end
            end

            %Update QQ
            QQ=updateq(QQ);

            if oldcountq<countq(QQ)
                Q=QQ;
                break
            end
        end

        if oldcountq<countq(Q)
            break
        end
    end
end

S=zeros(1,9);
for n=1:9
    for m=1:9
        if length(Q{n,m})==1
            S(n,m)=Q{n,m};
        end
    end
end

%%

%COUNTQ

function counter=countq(Q);

%Determine number of elements of Q that have length ==1
counter=0;
DIM=size(Q);
for n=1:DIM(1)
    for m=1:DIM(2)
        if length(Q{n,m})==1
            counter=counter+1;
        end
    end
end

%%

%UPDATEQ

function Q=updateq(Q)

for n=1:9
    for m=1:9
        if length(Q{n,m})>1
            V=[];
            H=[];
            for k=1:9
                if length(Q{k,m})==1
                    V=[V,Q{k,m}];
                end
                if length(Q{n,k})==1
                    H=[H,Q{n,k}];
                end
            end
            V=setxor(V,1:9);
            H=setxor(H,1:9);

            %Update Q{n,m}
            if (length(V)~=0) & (length(H)~=0)
                newQ=intersect(intersect(H,V),Q{n,m});


                if isempty(newQ)~=1
                    Q{n,m}=newQ;
                end

                if length(newQ)==1



                    %If intersection is a single integer, remove this
                    %integer from the other options in the V and H vectors
                    %crossing the element (n,m), then update Q{n,m} with
                    %intersection

                    for k=1:9
                        mQdiff=setdiff(Q{k,m},newQ);
                        nQdiff=setdiff(Q{n,k},newQ);
                        if (length(Q{k,m})>1) & (k~=n) & (isempty(mQdiff)~=1)
                            Q{k,m}=mQdiff;
                        end
                        if (length(Q{n,k})>1) & (k~=m) & (isempty(nQdiff)~=1)
                            Q{n,k}=nQdiff;
                        end
                    end

                end
            end
        end
    end
end

%End of code