%SUDOKU
% Solver for Sudoku puzzels. A solved Sudoku puzzle is a 9 x 9 matrix of
% integers from the set 1 ...9 such that no column or row has repeated
% elements. Also all 3 x 3 boxes (that completely divide the 9 x 9 matrix)
% must not contain repeated elements!
%
% LAST UPDATED by Andy French. 3rd August 2005.
%
% Syntax:  S=sudoku(P)
% P is a 9 x 9 matrix of integers 0 ... 9
% 0 denotes an unknown integer from the set 1...9 to be found.
% S is the solution matrix, with unknowns determined.
%
% Example:
% P=[...
%     8,5,0,0,0,6,0,0,0;....
%     0,0,3,1,8,0,7,4,0;...
%     0,0,7,0,0,0,0,0,5;...
%     5,0,0,0,6,0,0,3,0;
%     0,8,0,0,0,0,0,7,0;...
%     0,6,0,0,4,0,0,0,8;...
%     9,0,0,0,0,0,2,0,0;,...
%     0,2,5,0,7,3,1,0,0;
%     0,0,0,2,0,0,0,6,3];
%
% S=sudoku(P)
%
% Example 2: (Tough! Daily Telegraph #54)
% P=[...
%     7,0,9,0,0,2,0,3,0;....
%     0,0,0,0,1,9,0,0,6;...
%     4,0,0,0,3,0,1,0,0;...
%     0,0,1,0,0,0,0,6,0;
%     0,0,8,2,0,7,9,0,0;...
%     0,4,0,0,0,0,5,0,0;...
%     0,0,6,0,8,0,0,0,9;,...
%     9,0,0,6,2,0,0,0,0;
%     0,5,0,9,0,0,6,0,4];
% S=sudoku(P)


function S=sudoku(P)

%Update Q (options cell array)
Q=updateQ(P);

%Update P
P=updateP(Q);

%Step through Q and store vectors of non single integer elements.
%Store the lengths of these vectors
k=1;
for n=1:9
    for m=1:9
        if length(Q{n,m})>1
            Qstore{k}=Q{n,m};
            Qlength(k)=length(Qstore{k});
            Qn(k)=n;
            Qm(k)=m;
            k=k+1;
        end
    end
end
clear k n m;

%Systematically go through Qstore. Assume the correct integer is that of
%the current element in the Qstore

%Initialise iteration counter iter
iter=0;
while (countq(Q) > 0) & (iter<=100)

    %Update iteration counter and count total number of integers in Q
    iter=iter+1
    oldcountq=countq(Q);

    %Step through Q and store vectors of non single integer elements.
    %Store the lengths of these vectors
    k=1;
    for n=1:9
        for m=1:9
            if length(Q{n,m})>1
                Qstore{k}=Q{n,m};
                Qlength(k)=length(Qstore{k});
                Qn(k)=n;
                Qm(k)=m;
                k=k+1;
            end
        end
    end
    clear k n m;

    %Find smallest Qstore vector and choose the first integer. Update P
    %with this and compute options cell array Q. If the number of options
    %decreases, assign the number and iterate.

    ksmall=find(Qlength==min(Qlength));
    ksmall=ksmall(1);
    for z=1:Qlength(ksmall)
        PP=P;
        PP(Qn(ksmall),Qm(ksmall))=Qstore{ksmall}(z);
        QQ=updateQ(PP);
        num_options(z)=countq(QQ);
    end

    zmax=find(num_options==max(num_options));
    P(Qn(ksmall),Qm(ksmall))=Qstore{ksmall}(zmax(1));

    Q=updateQ(P);


end

%Form output S
S=updateP(Q);

%%

%COUNTQ

function counter=countq(Q);

%Determine number of options
counter=0;
DIM=size(Q);
for n=1:DIM(1)
    for m=1:DIM(2)
        if length(Q{n,m})~=1
            counter=counter+length(Q{n,m});
        end
    end
end

%%

%UPDATEP

function P=updateP(Q)

P=zeros(1,9);
for n=1:9
    for m=1:9
        if length(Q{n,m})==1
            P(n,m)=Q{n,m};
        end
    end
end

%%

%EMPTYQ

function L=emptyQ(Q)

%Check if any element of Q is empty. If this is the case a mistake has been
%made!
L=0;
for n=1:9
    for m=1:9
        if isempty(Q{n,m})==1
            L=L+1;
        end
    end
end
if L>0
    L=1;
end

%%

%UPDATEQ

function Q=updateQ(P)

%Initialise cell array of possible integers. The size of the array matches
%P, but in general elements can be vectors of several integers.
for n=1:9
    for m=1:9
        Q{n,m}=P(n,m);
    end
end

%Divide P into 3 x 3 squares. For each square determine the 's' set of
%possible integers.
for n=1:3
    for m=1:3

        %Extract non zero elements of P from square (n,m) to form 's' set
        p=zeros(3,3);
        p(:,:)=P(3*(n-1)+1 : 3*(n-1)+3 , 3*(m-1)+1 : 3*(m-1)+3);
        s=[];
        for k=1:9
            if p(k)~=0
                s=[s,p(k)];
            end
        end
        s=setxor(s,1:9);

        %Form V and H sets corresponding to the non zero elements of P in
        %the vertical columns and horizontal rows that overlap square n,m

        for i=1:3
            %V
            v=zeros(1,9);
            v(:)=P(:,3*(m-1)+i);
            V{i}=[];
            for k=1:9
                if v(k)~=0
                    V{i}=[V{i},v(k)];
                end
            end
            V{i}=setxor(V{i},1:9);

            %H
            h=zeros(1,9);
            h(:)=P(3*(n-1)+i,:);
            H{i}=[];
            for k=1:9
                if h(k)~=0
                    H{i}=[H{i},h(k)];
                end
            end
            H{i}=setxor(H{i},1:9);
        end

        %For each of the zeros of p, determine possible integers that are
        %in the corresponding H, V & s set
        for x=1:3
            for y=1:3
                if p(x,y)==0
                    %H,V,s intersection
                    HVs=intersect(intersect(H{x},V{y}),s);

                    %If HVs is a single element, remove this from s
                    if length(HVs)==1
                        rm_index=find(s==HVs);
                        new_s=[];
                        for t=1:length(s)
                            if t~=rm_index
                                new_s=[new_s,s(t)];
                            end
                        end
                        s=new_s;
                    end

                    %Set vector element of Q
                    Q{3*(n-1)+x, 3*(m-1)+y}=HVs;
                end
            end
        end
    end
end

%End of code