%sudoku_samecandidates
% If a sudoku S,V or H region has k identical candidates, then these values
% cannot be candidates for other values of the S,V or H regions.
%
% LAST UPDATED by Andy French 30th January 2010

function [C,x] = sudoku_samecandidates( x, C, na )

%Check whether sudoku has been solved
if sudoku_issolved(x)
    return
end

%%%% COLUMNS %%%%

%Step through columns of Candidates array C and determine any instances of
%K equal elements of length K, for values of K between 2 (i.e. not a
%deduction) and na-1 (i.e. there are possibly some remaining cells in the
%column which can have candidate values removed)
for n=1:na
    
    %Initialise vector of candidate elements checked
    checked = 1:na;
    
    %Step through candidate elements and check for the required repetitions
    while length(checked)>1
        %Count numbers of elements which match the first element of set
        %indexed by array checked
        count = 0;
        matches = checked(1);
        for m=2:length(checked)
            %Determine if subsequent elements of C{ checked(m),n } match the first one indexed
            %by checked(1)
            if isequal( C{ checked(m),n }, C{ checked(1),n } ) 
                count = count + 1;
                matches = [matches,checked(m)];
            end
        end
        
        %Does count equal the number of elements in C{ checked(1), n } ?
        if count == length( C{ checked(1),n } )
            %Remove values of C{ checked(1) from non matches
            nonmatches = setdiff( 1:na, matches );
            for z=1:length(nonmatches)
                C{ nonmatches(z), n } = setdiff( C{ nonmatches(z), n }, C{ checked(1),n } );
            end
        end
        
        %Remove matches from checked elements
        checked = setdiff( checked, matches );
    end    
end

%%%% ROWS %%%%

%Step through rows of Candidates array C and determine any instances of
%K equal elements of length K, for values of K between 2 (i.e. not a
%deduction) and na-1 (i.e. there are possibly some remaining cells in the
%row which can have candidate values removed)
for n=1:na
    
    %Initialise vector of candidate elements checked
    checked = 1:na;
    
    %Step through candidate elements and check for the required repetitions
    while length(checked)>1
        %Count numbers of elements which match the first element of set
        %indexed by array checked
        count = 0;
        matches = checked(1);
        for m=2:length(checked)
            %Determine if subsequent elements of C{ checked(m),n } match the first one indexed
            %by checked(1)
            if isequal( C{ n, checked(m) }, C{ n, checked(1) } ) 
                count = count + 1;
                matches = [matches,checked(m)];
            end
        end
        
        %Does count equal the number of elements in C{ n, checked(1) } ?
        if count == length( C{ n, checked(1) } )
            %Remove values of C{ checked(1) from non matches
            nonmatches = setdiff( 1:na, matches );
            for z=1:length(nonmatches)
                C{ n, nonmatches(z) } = setdiff( C{ n, nonmatches(z) }, C{ n, checked(1) } );
            end
        end
        
        %Remove matches from checked elements
        checked = setdiff( checked, matches );
    end    
end

%%%% SQUARE SETS %%%%

%Step through square set of Candidates array C and determine any instances of
%K equal elements of length K, for values of K between 2 (i.e. not a
%deduction) and na-1 (i.e. there are possibly some remaining cells in the
%column which can have candidate values removed)
for n=1:na
    
    %Initialise vector of candidate elements checked
    checked = 1:na;
    
    %Determine sudoku indices associated with square set n
    [rows_S_n,cols_S_n] = sudoku_indices_from_S(n,na);
    
    %Step through candidate elements and check for the required repetitions
    while length(checked)>1
        %Count numbers of elements which match the first element of set
        %indexed by array checked
        count = 0;
        matches = checked(1);
        for m=2:length(checked)
            %Determine if subsequent elements of C{ checked(m),n } match the first one indexed
            %by checked(1)
            if isequal( C{ rows_S_n( checked(m) ),cols_S_n( checked(m) ) },...
                    C{ rows_S_n( checked(1) ),cols_S_n( checked(1) ) } )
                count = count + 1;
                matches = [matches,checked(m)];
            end
        end
        
        %Does count equal the number of elements in C{ rows_S_n( checked(1) ),cols_S_n( checked(1) ) } ?
        if count == length( C{ rows_S_n( checked(1) ),cols_S_n( checked(1) ) } )
            %Remove values of C{ rows_S_n( checked(1) ),cols_S_n( checked(1) ) } from non matches
            nonmatches = setdiff( 1:na, matches );
            for z=1:length(nonmatches)
                C{ rows_S_n( nonmatches(z) ),cols_S_n( nonmatches(z) ) } =...
                    setdiff( C{ rows_S_n( nonmatches(z) ),cols_S_n( nonmatches(z) ) },...
                    C{ rows_S_n( checked(1) ),cols_S_n( checked(1) ) } );
            end
        end
        
        %Remove matches from checked elements
        checked = setdiff( checked, matches );
    end    
end

%End of code