%transformer_gui
% Model of a transformer consisting of an AC voltage source in series
% with a load of resistance r and a solenoid of inductance Lp. This is
% connected via a magnetic C-core to another solenoid of inductance Ls in
% series with a load of resistance R. The voltage across this is Vs.
%
% It is assumed the corss sectional areas (Ar) of both primary and secondary
% solenoids are the same. Hence:
%
% Lp/Ls = (np/ns)^2   since Lp = mu*mu0*np^2*Ar
%
% Let Vp, Vs, Ip, Is be the real parts of the following complex expressions
%
% Vp = V0*exp(i*w*t)
% Ip = (V0/r)*A*exp(i*w*t)
% Is = (V0/R)*B*exp(i*w*t)
% Vs = Is*R
%
% Note A,B are dimensionless complex numbers
%
% LAST UPDATED by Andy French November 2016

function varargout = transformer_gui(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @transformer_gui_OpeningFcn, ...
    'gui_OutputFcn',  @transformer_gui_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% transformer_gui opening function. This executes just before transformer_gui is made visible.
function transformer_gui_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for transformer
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;
d.legend = [];

%Center figure
figcentre( d.h.FIGUREtransformer );

%Set defaults
transformer_defaults

%Update transformer
transformer_update;

%%

% Default parameters
function transformer_defaults
global d

%Primary load r /ohms
d.r = 10;
d.r_max = 100;

%Secondary load R /ohms
d.R = 100;
d.R_max = 2000;

%Primary turns Np
d.Np = 100;
d.Np_max = 1000;

%Secondary turns Ns
d.Ns = 200;
d.Ns_max = 1000;

%Coupling constant k
d.k = 0.9;
d.k_max = 1;

%Core relative permeability mu
d.mu = 30;

%Permeability of free space
d.mu0 = 4*pi*1e-7;

%Cross sectional area of transformer coils /m^2
d.A = 0.1;

%Coil lengths /m
d.l = 0.1;

%Primary coil AC voltage amplitude /volts
d.V0 = 10;

%Max input AC voltage frequency /Hz (for plot)
d.fmax = 50;
d.f = d.fmax/2;

%Maximum number of computation points
d.N = 1000;

%Fontsize for graphs
d.fsize = 12;

%Number of significant figures for graph title
d.sfs = 3;

%Default is that fixed axes scale is off
d.fix_scale = 0;

%Plot externally and save .PNG
d.plot_and_save_PNG = 0;

%Set GUI parameters
set( d.h.EDITr,'string',num2str(d.r) );
set( d.h.EDITR,'string',num2str(d.R) );
set( d.h.EDITNp,'string',num2str(d.Np) );
set( d.h.EDITNs,'string',num2str(d.Ns) );
set( d.h.EDITk,'string',num2str(d.k) );
set( d.h.EDITrmax,'string',num2str(d.r_max) );
set( d.h.EDITRmax,'string',num2str(d.R_max) );
set( d.h.EDITNpmax,'string',num2str(d.Np_max) );
set( d.h.EDITNsmax,'string',num2str(d.Ns_max) );
set( d.h.EDITkmax,'string',num2str(d.k_max) );
set( d.h.EDITmu,'string',num2str(d.mu) );
set( d.h.EDITA,'string',num2str(d.A) );
set( d.h.EDITV0,'string',num2str(d.V0) );
set( d.h.EDITl,'string',num2str(d.l) );
set( d.h.EDITfmax,'string',num2str(d.fmax) );
set( d.h.EDITf,'string',num2str(d.fmax/2) );
set( d.h.SLIDERf,'value',0.5 );
set( d.h.SLIDERr,'value',d.r/d.r_max );
set( d.h.SLIDERR,'value',d.R/d.R_max );
set( d.h.SLIDERNp,'value',d.Np/d.Np_max);
set( d.h.SLIDERNs,'value',d.Ns/d.Ns_max);
set( d.h.SLIDERk,'value',d.k/d.k_max);
set( d.h.POPUPMENUchoosegraph,'string',...
    {'|Vs/V0|','Phase of Vs','Current magnitude',...
    'Current phase','Power',...
    'Primary voltage phasor diagram',...
    'Secondary voltage phasor diagram'
    },'value',1 );

%%

%% Diffractor output function. Outputs from this function are returned to the command line.
function varargout = transformer_gui_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Toggle fix axes scale for plot
function TOGGLEfixaxesscale_Callback(hObject, eventdata, handles)
global d
if d.fix_scale==1
    d.fix_scale = 0;
else
    d.fix_scale = 1;
end
transformer_update

%%

%% Save PNG of graph
function PUSHsavepng_Callback(hObject, eventdata, handles)
global d
d.plot_and_save_PNG = 1;
transformer_update;
d.plot_and_save_PNG = 0;

%%

%% Frequency selection (for Phasor mode only) slider
function SLIDERf_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERf,'value');

%Determine value of f and update associated edit box
d.f = v * d.fmax;
set(d.h.EDITf,'string',num2str(d.f));

%Update GUI
transformer_update

%%

%% Primary load r slider
function SLIDERr_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERr,'value');

%Determine value of r and update associated edit box
d.r = v * d.r_max;
set(d.h.EDITr,'string',num2str(d.r));

%Update GUI
transformer_update

%%

%% Secondary load R slider
function SLIDERR_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERR,'value');

%Determine value of R and update associated edit box
d.R = v * d.R_max;
set(d.h.EDITR,'string',num2str(d.R));

%Update GUI
transformer_update

%%

%% Primary turns Ns slider
function SLIDERNp_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERNp,'value');

%Determine value of Np and update associated edit box
d.Np = round(v * d.Np_max);
set(d.h.EDITNp,'string',num2str(d.Np));

%Update GUI
transformer_update

%%

%% Secondary turns Ns slider
function SLIDERNs_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERNs,'value');

%Determine value of Ns and update associated edit box
d.Ns = round(v * d.Ns_max);
set(d.h.EDITNs,'string',num2str(d.Ns));

%Update GUI
transformer_update

%%

%% Coupling constant k slider
function SLIDERk_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERk,'value');

%Determine vale of k and update associated edit box
d.k = v * d.k_max;
set(d.h.EDITk,'string',num2str(d.k));

%Update GUI
transformer_update

%%

%% Frequency select edit box (used in Phasor plot only)
function EDITf_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.f = abs( str2num(get(handles.EDITf,'string')) );

%Update to make sure only positive values are allowed
set( handles.EDITf,'string', num2str(d.f) );

%Update slider
v = d.f/d.fmax;
set(d.h.SLIDERf,'value',v);

%Update GUI
transformer_update

%%

%% Primary load r edit box
function EDITr_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.r = round( abs( str2num(get(handles.EDITr,'string')) ) );

%Update to make sure only positive values are allowed
set( handles.EDITr,'string', num2str(d.r) );

%Update slider
v = d.r/d.r_max;
set(d.h.SLIDERr,'value',v);

%Update GUI
transformer_update

%%

%% Secondary load R edit box
function EDITR_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.R = round( abs( str2num(get(handles.EDITR,'string')) ) );

%Update to make sure only positive values are allowed
set( handles.EDITR,'string', num2str(d.R) );

%Update slider
v = d.R/d.R_max;
set(d.h.SLIDERR,'value',v);

%Update GUI
transformer_update

%%

%% Primary turns Ns edit box
function EDITNp_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.Np = round( abs( str2num(get(handles.EDITNp,'string')) ) );

%Update to make sure only positive integer values are allowed
set( handles.EDITNp,'string', num2str(d.Np) );

%Update slider
v = d.Np/d.Np_max;
set(d.h.SLIDERNp,'value',v);

%Update GUI
transformer_update

%%

%% Secondary turns Ns edit box
function EDITNs_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.Ns = round( abs( str2num(get(handles.EDITNs,'string')) ) );

%Update to make sure only positive integer values are allowed
set( handles.EDITNs,'string', num2str(d.Ns) );

%Update slider
v = d.Ns/d.Ns_max;
set(d.h.SLIDERNs,'value',v);

%Update GUI
transformer_update

%%

%% Coupling constant k slider edit box
function EDITk_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.k = abs( str2num(get(handles.EDITk,'string')) );
if d.k > 1
    d.k = 1;
end

%Update to make sure only positive values are allowed
set( handles.EDITk,'string', num2str(d.k) );

%Update slider
v = d.k/d.k_max;
set(d.h.SLIDERk,'value',v);

%Update GUI
transformer_update

%%
%% Primary load r max edit box
function EDITrmax_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.r_max = abs( str2num(get(handles.EDITrmax,'string')) ) ;

%Update to make sure only positive values are allowed
set( handles.EDITrmax,'string', num2str(d.r_max) );

%Update slider
v = d.r/d.r_max;
set(d.h.SLIDERr,'value',v);

%Update GUI
transformer_update

%%

%% Secondary load R max edit box
function EDITRmax_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.R_max = abs( str2num(get(handles.EDITRmax,'string')) ) ;

%Update to make sure only positive values are allowed
set( handles.EDITRmax,'string', num2str(d.R_max) );

%Update slider
v = d.R/d.R_max;
set(d.h.SLIDERR,'value',v);

%Update GUI
transformer_update

%%

%% Primary turns Ns max edit box
function EDITNpmax_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.Np_max = round( abs( str2num(get(handles.EDITNpmax,'string')) ) );

%Update to make sure only positive integer values are allowed
set( handles.EDITNpmax,'string', num2str(d.Np_max) );

%Update slider
v = d.Np/d.Np_max;
set(d.h.SLIDERNp,'value',v);

%Update GUI
transformer_update

%%

%% Secondary turns Ns max edit box
function EDITNsmax_Callback(hObject, eventdata, handles)
global d

%Get data from edit box
d.Ns_max = round( abs( str2num(get(handles.EDITNsmax,'string')) ) );

%Update to make sure only positive integer values are allowed
set( handles.EDITNsmax,'string', num2str(d.Ns_max) );

%Update slider
v = d.Ns/d.Ns_max;
set(d.h.SLIDERNs,'value',v);

%Update GUI
transformer_update

%%

%% Coupling constant k max edit box
function EDITkmax_Callback(hObject, eventdata, handles)
global d

%Get data from edit box. Force to be in the range 0 <= kmax <=1
d.k_max = abs( str2num(get(handles.EDITkmax,'string')) );
if d.k_max > 1
    d.k_max = 1;
end

%Update to make sure only positive values are allowed
set( handles.EDITkmax,'string', num2str(d.k_max) );

%Update slider
v = d.k/d.k_max;
set(d.h.SLIDERk,'value',v);

%Update GUI
transformer_update

%%

%% Core permeability edit box
function EDITmu_Callback(hObject, eventdata, handles)
global d
d.mu = str2num( get(handles.EDITmu,'string') );

%Update GUI
transformer_update

%%

%% Primary voltage edit box
function EDITV0_Callback(hObject, eventdata, handles)
global d
d.V0 = str2num( get(handles.EDITV0,'string') );

%Update GUI
transformer_update

%%

%% Coil cross sectional area edit box
function EDITA_Callback(hObject, eventdata, handles)
global d
d.A = str2num( get(handles.EDITA,'string') );

%Update GUI
transformer_update

%%

%% Coil length edit box
function EDITl_Callback(hObject, eventdata, handles)
global d
d.l = str2num( get(handles.EDITl,'string') );

%Update GUI
transformer_update

%%

%% Maximum frequency /Hz for graph
function EDITfmax_Callback(hObject, eventdata, handles)
global d
d.fmax = str2num( get(d.h.EDITfmax,'string') );
d.f = d.fmax/2;
set( d.h.EDITf,'string',num2str(d.f) );
set( d.h.SLIDERf,'value',0.5);

%Update GUI
transformer_update

%%

%% Choose plot pop-up-menu
function POPUPMENUchoosegraph_Callback(hObject, eventdata, handles)
global d
d.fix_scale = 0;
set(d.h.TOGGLEfixaxesscale,'value',0);
transformer_update

%%

%Update GUI
function transformer_update
global d

%Determine inductances /H
d.Lp = d.mu*d.mu0*(d.Np^2)*d.A/d.l;
d.Ls = d.mu*d.mu0*(d.Ns^2)*d.A/d.l;

%Update GUI with inductances /H
set( d.h.EDITLp,'string',num2str(d.Lp));
set( d.h.EDITLs,'string',num2str(d.Ls));

%Define frequencies /Hz
f = linspace(0,d.fmax,d.N);

%Evaluate transformer model at frequencies f
[Is,Ip,Pp,Ps,d.M,e,ep,es,emp,ems,Vp,Vs] = transformer_model(f,d.k,d.Lp,d.Ls,d.r,d.R,d.V0);
set( d.h.EDITM,'string',num2str(d.M));

%Define filename, if a PNG file is to be created
png_filename = ['r=',num2str(d.r),' R=',num2str(d.R),' Np=',num2str(d.Np),...
    ' Ns=',num2str(d.Ns),' k=',num2str(d.k),' Lp=',num2str(d.Lp),' Ls=',num2str(d.Ls)];
png_filename = strrep( png_filename,'.','p');

%What to do if plotting to a new figure window
if d.plot_and_save_PNG==1
    %Create a new figure window and make this active
    fig = figure;
    figure(fig);
else
    %Point to GUI axes
    axes(d.h.AXEStransformer);
    cla;
    delete(d.legend);
    d.legend = [];
end

%Depending on option set, plot graph
options = get(d.h.POPUPMENUchoosegraph,'string');
option = options{ get(d.h.POPUPMENUchoosegraph,'value') };
phasor = 0;
switch option
    case '|Vs/V0|'
        plot(f,abs(Vs/d.V0));
        xlabel('Frequency /Hz','fontsize',d.fsize)
        ylabel('|Vs/V0|','fontsize',d.fsize)
        set(gca,'fontsize',d.fsize)
        title(['Magnitude of Vs/V0. Ns/Np = ',num2str(d.Ns/d.Np)],'fontsize',d.fsize)
        png_filename = ['Mag Vs to Vp ',png_filename];
    case 'Phase of Vs'
        plot(f,angle(Vs/d.V0));
        xlabel('Frequency /Hz','fontsize',d.fsize)
        ylabel('Phase /radians','fontsize',d.fsize)
        set(gca,'fontsize',d.fsize)
        title('Phase of Vs/V0 /radians','fontsize',d.fsize)
        png_filename = ['Vs to V0 Phase ',png_filename];
    case 'Current magnitude'
        plot(f,abs(Ip),'r');
        hold on
        plot(f,abs(Is),'b');
        d.legend=legend({'Primary','Secondary'},'fontsize',d.fsize);
        xlabel('Frequency /Hz','fontsize',d.fsize)
        ylabel('Current /A','fontsize',d.fsize)
        title('Current magnitude in coils','fontsize',d.fsize)
        png_filename = ['Current mag ',png_filename];
    case 'Current phase'
        plot(f,angle(Ip),'r');
        hold on
        plot(f,angle(Is),'b');
        d.legend=legend({'Primary','Secondary'},'fontsize',d.fsize);
        xlabel('Frequency /Hz','fontsize',d.fsize)
        ylabel('Phase /radians','fontsize',d.fsize)
        title('Current phase in coils','fontsize',d.fsize)
        png_filename = ['Current phase ',png_filename];
    case 'Power'
        plot(f,Pp,'r');
        hold on
        plot(f,Ps,'b');
        d.legend=legend({'Primary','Secondary'},'fontsize',d.fsize);
        xlabel('Frequency /Hz','fontsize',d.fsize)
        ylabel('Power /watts','fontsize',d.fsize)
        title('Power dissipated in loads','fontsize',d.fsize)
        png_filename = ['Power ',png_filename];
        
    case 'Primary voltage phasor diagram'
        phasor = 1;
        
        %Primary coil EMF sum and load PD
        
        %Evaluate transformer model at frequency d.f
        [Is,Ip,Pp,Ps,M,e,ep,es,emp,ems,Vp,Vs] = transformer_model(d.f,d.k,d.Lp,d.Ls,d.r,d.R,d.V0);
        
        %Plot voltage phasor - primary coil
        x = real(e);
        y = imag(e);
        plot([0,x],[0,y],'r'); hold on;
        xx = x;
        yy = y;
        x = xx + real(ep);
        y = yy + imag(ep);
        plot([xx,x],[yy,y],'g'); hold on;
        xx = x;
        yy = y;
        x = xx + real(emp);
        y = yy + imag(emp);
        plot([xx,x],[yy,y],'b'); hold on;
        plot([0,real(Vp)],[0,imag(Vp)],'k');
        xlabel('Volts (real)','fontsize',d.fsize)
        ylabel('Volts (imaginary)','fontsize',d.fsize)
        title('Primary coil phasor','fontsize',d.fsize)
        d.legend=legend({'EMF p','back EMF p','M EMF','PD p'},'fontsize',...
            d.fsize,'location','eastoutside');
        png_filename = ['Primary phasor ',png_filename];
        
    case 'Secondary voltage phasor diagram'
        phasor = 1;
        
        %Secondary coil EMF sum and load PD
        
        %Evaluate transformer model at frequency d.f
        [Is,Ip,Pp,Ps,M,e,ep,es,emp,ems,Vp,Vs] = transformer_model(d.f,d.k,d.Lp,d.Ls,d.r,d.R,d.V0);
        
        %Plot voltage phasor - secondary coil
        plot([0,real(ems)],[0,imag(ems)],'r'); hold on;
        plot([real(ems),real(ems)+real(es)],...
            [imag(ems),imag(ems)+imag(es)],'g'); hold on;
        plot([0,real(Vs)],[0,imag(Vs)],'k');
        xlabel('Volts (real)','fontsize',d.fsize)
        ylabel('Volts (imaginary)','fontsize',d.fsize)
        title('Secondary coil phasor','fontsize',d.fsize)
        d.legend=legend({'M EMF','back EMF s','PD s'},'fontsize',d.fsize,...
            'location','eastoutside');
        png_filename = ['Secondary phasor ',png_filename];
end

%Set axes scale, grid and fontsize
if phasor ==0
    if d.fix_scale == 0
        axis tight
        d.yscale = get(gca,'ylim');
    else
        axis manual
        set(gca,'ylim',d.yscale);
    end
    xlim( [0,d.fmax] );
else
    if d.fix_scale == 0
        axis tight
        d.yscale = get(gca,'ylim');
        d.xscale = get(gca,'xlim');
    else
        axis manual
        set(gca,'xlim',d.xscale,'ylim',d.yscale);
    end
end
grid on
set(gca,'fontsize',d.fsize)

%Save externally plotted graph, if option set
if d.plot_and_save_PNG==1
    print( fig,png_filename,'-dpng','-r300');
    close(fig);
    figure(d.h.FIGUREtransformer);
    axes(d.h.AXEStransformer);
    d.plot_and_save_PNG=0;
    d.legend = [];
    transformer_update
end

%%

%Transformer model
function [Is,Ip,Pp,Ps,M,e,ep,es,emp,ems,Vp,Vs] = transformer_model(f,k,Lp,Ls,r,R,V0)

%Determine mutual inductance /H
M = k*sqrt(Lp*Ls);

%Determine angular frequency w
w = 2*pi*f;

%Determine complex numbers A and B
A = ( 1 + Lp*1i*w/r + (M^2)*(w.^2)./(r*R + r*Ls*1i*w) ).^-1;
B = M*R*1i*w.*A./(r*R + r*Ls*1i*w);

%Determine complex values of Vs, Is, Ip  (i.e. relative to cosine wave
%oscillating at f Hz)
e = V0;
Ip = (V0/r)*A;
Is = (V0/R)*B;
ep = -Lp*1i*w.*Ip;
es = -Ls*1i*w.*Is;
emp = M*1i*w.*Is;
ems = M*1i*w.*Ip;
Vp = Ip*r;
Vs = Is*R;

%Determine power dissipated in loads
Pp = 0.5*real( Vp.*conj(Ip) );
Ps = 0.5*real( Vs.*conj(Is) );

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code






