%multi_value_trig
% Function which finds multiple solutions of trignometric equations of the
% form t( f(x) ) = k where t is sin, cos or tan.
% NOTE f(x) must not have any multiple values over the range of x chosen.
%
% LAST UPDATED by Andy French Sept 2012
%
% Syntax: [x,xx,y,yy,f] = multi_value_trig( t, fx, gx, k, xok, N )
%
% t       Trig function: either 'sin', 'cos' or 'tan'
% fx      Argument of trig function t( f(x) ) IN RADIANS
% gx      Inverse function of argument of trig function t( f(x) );
% k       RHS of equation to be solved t( f(x) ) = k
% xok     Define range IN RADIANS of allowed solutions
% N       Number of data points
%
% x       Allowed range of x, N linearly spaced points
% xx      Vector of solutions in radians
% yy      yy = t( f(xx) )
% y       y = t( f(x) )
% f       f(x)

function [x,xx,y,yy,f] = multi_value_trig( t, fx, gx, k, xok, N )

%Define range of x and fx
x = linspace( xok(1), xok(2), N);
f = F(x,fx);

%Find 'basic' solutions and then compute multiple values

%Work out deviations from 0, 270, 90 or -90
if strcmp( t,'sin' )
    y = sin(f);
    b = asin(k);
    df = 2*pi;
    if abs(k) > 1
        b = NaN;  %i.e. ignore complex solutions
        f0=NaN;
    elseif k<0
        b = 90*pi/180 - abs(b);
        f0 = -pi/2;
    else
        b = 90*pi/180 - b;
        f0 = pi/2;
    end
elseif strcmp( t,'cos' )
    y=cos(f);
    b = acos(k);
    df = 2*pi;
    if abs(k) > 1
        b = NaN;  %i.e. ignore complex solutions
        f0 = NaN;
    elseif k<0
        b = pi - abs(b);
        f0 = pi;
    else
        f0 = 0;
    end
elseif strcmp( t,'tan' )
    y = tan(f);
    b = atan2(k,1);
    df = pi;
    f0=0;
end

%Compute multiple values and add these to array f
f = f0+b;
if strcmp( t,'tan' )==0
    f = [f,f0-b];
end
n = 1;
ff = f0+b;
while ( ff> F(xok(1),fx) )
    if strcmp( t,'tan' )==0
        ff = f0 -b - n*df;
        f = [ff,f];
    end
    ff = f0 +b - n*df;
    f = [ff,f];
    n = n + 1;
end
n=1;
ff = f0+b;
while  ( ff < F(xok(2),fx) );
    if strcmp( t,'tan' )==0
        ff = f0 -b + n*df;
        f = [f,ff];
    end
    ff = f0 +b + n*df;
    f = [f,ff];
    n = n + 1;
end
f( f < F(xok(1),fx) ) = [];
f( f > F(xok(2),fx) ) = [];

%Convert solutions to x not f(x)
xx = sort( F(f,gx) );

%Determine associated values of y = t( f(xx) )
eval( ['yy = ',t,'(F(xx,fx));'])

%%

function y = F(x,fx)
eval(['y=',fx,';'])
y = real(y);

%End of code