%waveformer
% A GUI which enables the pulse, which constitutes a periodic waveform, to
% be designed using mouse clicks. The program will connect mouse clicks
% with cubic splines to produce a smooth function at the desired smaple
% rate. The waveform repetition frequency is set, whose reciprocal defines
% the period of the waveform. At least two points define a pulse. (You
% won't be able to delete any more when you get down to two!)
%
% Playback of the waveform is enabled by this GUI, and waveforms can be
% loaded and saved.
%
% LAST UPDATED by Andy French May 2014

function varargout = waveformer(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @waveformer_OpeningFcn, ...
    'gui_OutputFcn',  @waveformer_OutputFcn, ...
    'gui_LayoutFcn',  [], ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

% End initialization code - DO NOT EDIT

%%

%%% OPENING FUNCTION & DEFAULTS %%%

% Set default parameters
function [g,d] = set_defaults

%Waveform repetition frequency /Hz
d.WRF = 500;

%Waveform f(t) function in terms of t and period T
d.fvst = 'f = ones(size(t))/T';

%Waveform period /s
d.period = 1/d.WRF;

%Waveform sample rate /Hz
g.fs = 44000;

%Graph y axis minimum
g.smax = 1;

%Graph y axis maximum
g.smin = -1;

%Graph y axis tick separation
g.stick = 0.2;

%Graph time axis maximum /ms
g.tmax = 2;

%Graph time axis minimum /ms
g.tmin = 0;

%Graph time axis tick separation /ms
g.ttick = 0.1;

%Played sound duration /s
g.duration = 2;

%Graph edit mode toggle
g.edit_mode = 0;

%Number of points for waveform
d.N = 100;

%Number of points for (interpolated) graph between ranges
g.Ndisplay = 1000;

%Set default pulse shape
d.t = linspace( 0, d.period, d.N );
d.s = sin( 2*pi*d.t/ d.period );

%%

% Opening function
function waveformer_OpeningFcn(hObject, eventdata, handles, varargin)

%Create a global variable d which will be a structured array and will store
%the GUI data
global d g h;

%Get default parameters
[g,d] = set_defaults;

%Store the GUI handles
h = handles;

%Default 'has mouse been clicked' toggle
h.mouse_clicked = 0;

%Plot graph
plot_graph( h.AXESgraph, h.FIGUREwaveformer )

%Update the GUI with default parameters
gui_update;

%%

% Output function
function varargout = waveformer_OutputFcn(hObject, eventdata, handles)

%%

%%% GUI UPDATE %%%

% Get GUI data
function get_gui_data
global d g h

%Get parameters from GUI and store these in global array d
% Note times are stored in seconds, and frequencies in Hz
% apart from graph limits, which are in ms
d.WRF = str2num( get(h.EDITWRF,'string') );
d.period = 1/d.WRF;
g.fs = 1000*str2num( get(h.EDITfs,'string') );
g.smax = str2num( get(h.EDITsmax,'string') );
g.smin = str2num( get(h.EDITsmin,'string') );
g.stick = str2num( get(h.EDITstick,'string') );
g.tmax = str2num( get(h.EDITtmax,'string') );
g.tmin = str2num( get(h.EDITtmin,'string') );
g.ttick = str2num( get(h.EDITttick,'string') );
g.duration = str2num( get(h.EDITduration,'string') );
g.edit_mode = get(h.TOGGLEeditwaveform,'value');
d.N = str2num( get(h.EDITN,'string') );
d.fvst = get( h.EDITfvst,'string' );

%%

% Plot graph. Initialize graph from parameters in structures d and g
function plot_graph( ax, fig )
global d g h

%Point to required figure and axes and clear
figure(fig); axes(ax); cla;

%Plot pulse shape
t = linspace( g.tmin,g.tmax,g.Ndisplay );
ss = interp1( d.t*1000, d.s, t );
h.points = plot( d.t*1000, d.s, 'bo' ); hold on;
h.trace = plot( t, ss, 'b' );
grid on

%Set graph annotation
xlabel('Time /ms','fontsize',12 )
ylabel('Pulse amplitude','fontsize',12 )
title('Pulse shape','fontsize',12 )
axis manual

%%

% Update GUI from parameters in structures d and g
function gui_update
global d g h

%Update edit boxes etc
set(h.EDITWRF,'string',num2str( d.WRF) );
set(h.EDITfs,'string',num2str( g.fs/1000 ) );
set(h.EDITsmax,'string',num2str( g.smax ) );
set(h.EDITsmin,'string',num2str( g.smin ) );
set(h.EDITstick,'string',num2str( g.stick ) );
set(h.EDITtmax,'string',num2str( g.tmax ) );
set(h.EDITtmin,'string',num2str( g.tmin ) );
set(h.EDITttick,'string',num2str(g.ttick) )
set(h.EDITduration,'string',num2str(g.duration) );
set(h.TOGGLEeditwaveform,'value', g.edit_mode );
set(h.EDITN,'string',num2str(d.N) );
set(h.EDITfvst,'string',d.fvst);

%Evaluate frequency formula
fvst = get(h.EDITfvst,'string');
if ~strcmp( get(h.EDITfvst,'string'), 'user modified' )
    
    %Set time points for pulse /s
    d.t = linspace( 0, d.period, d.N );
    
    %Make copies of time array and period /ms
    t = d.t;
    T = d.period;
    
    %Evaluate f vs t function
    eval( [ fvst,';'] );
    
    %Determine phase of waveform vs time
    [Y_pp,phase]=calculus(t,2*pi*f,t,'integ','spline',0);
    
    %Determine waveform and update graph
    d.s = sin( phase );
    %d.s = sin( 2*pi*f.*d.t );
end

%Update plot points (blue circles)
set(h.points, 'xdata', d.t*1000, 'ydata', d.s);
if get( h.CHECKhidepoints,'value' ) == 1
    set( h.points,'marker','none' );
else
    set( h.points,'marker','o' );
end

%Interpolate data points and plot trace
t = linspace( g.tmin,g.tmax,g.Ndisplay );
[tp,i] = unique( d.t*1000 );
ss = interp1( tp, d.s(i), t, 'pchip' );
set( h.trace, 'xdata', t, 'ydata', ss );

%Update axis of graph
set( h.AXESgraph,'xlim',[g.tmin,g.tmax],'ylim',[g.smin,g.smax],...
    'xtick', g.tmin: g.ttick : g.tmax, 'ytick',g.smin: g.stick : g.smax );

%%

%%% INPUT & OUTPUT %%%

% Save waveform to a .mat file
function PUSHsave_Callback(hObject, eventdata, handles)
global d g

%Assemble data structure to save
S.d = d;

%Store GUI data as well
S.g = g;

%Save array d as a .mat file
filename = ['waveform ',strrep(datestr(now),':','-')];
save( filename, '-struct', 'S' );

%%

% Load waveform push button
function PUSHloadwaveform_Callback(hObject, eventdata, handles)
global d g

%Load waveform, replacing parameters in array d
[filename, pathname, filterindex] = uigetfile( '*.mat','Choose waveform file to load ...');
if ~isempty(filename)
    S = load( [pathname,'\',filename] );
    d = S.d;
    g = S.g;
end

%Update GUI
gui_update

%%

% Save current graph view as a 300dpi PNG file
function PUSHpng_Callback(hObject, eventdata, handles)
global d g h

%Temporarily create a new figure
fig = figure; ax = axes;
plot_graph( ax, fig )

%Plot graph as a .PNG
filename = ['waveform ',strrep(datestr(now),':','-')];
print( fig, filename, '-dpng', '-r300' );

%Close figure
close(fig);

%%

%%% SOUND %%%

% Play waveform for the duration set in the 'Sound duration' edit box
function PUSHplay_Callback(hObject, eventdata, handles)
global d g

%Determine how many pulses fit into the specified duration
N = ceil( g.duration/d.period );

%Repeat the pulse over the sound duration
s = repmat( d.s,[1,N] );

%Work out the coresponding times /s
t=[];
for n=1:N
    t = [t, d.t + (n-1)*d.period ];
end

%Sample the pulse at the set sample rate (units are s and Hz)
tt = 0 : 1/g.fs : g.duration;

%Interpolate data points
[tp,i] = unique(t);
s = interp1( tp, s(i), tt, 'pchip' );

%Filter out high frequencies   *** NEEDS WORK! ***
z = 0.5;
s = filter(z, [1 z-1], s);

%Define a smooth amplitude envelope to prevent nasty harmonics resulting
%from starting and ending clips of the loudspeaker
a = exp( -linspace(-1,1,length(s)).^2 );
s = a.*s;

%Replicate pulse and play sound
sound( 0.2*[s.',s.'] , g.fs  );

%%

% Played sound duration edit box
function EDITduration_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

%%% DYNAMIC GRAPH EDITING WITH THE MOUSE %%%

% Callback which runs when the mouse button is released in the figure window
function MouseClickUp(hObject, eventdata, handles)
global h

%Update mouse click toggle and graph
h.mouse_clicked = 0;
gui_update;

%%

% Callback which runs when the mouse is clicked in the figure window
function MouseClickDown(hObject, eventdata, handles)
global d g h

%Check if in edit mode
if g.edit_mode == 1
    
    %Change f vs t to be 'user modified'
    d.fvst = 'user modified';
    set( h.EDITfvst,'string', d.fvst );
    
    %Update mouse click toggle
    h.mouse_clicked = 1;
    
    %Determine whether click is left or 'something else' (e.g. right)
    %'Something else' will delete the current point and not allow any point
    %movement
    if strcmp( get( h.FIGUREwaveformer, 'SelectionType'), 'normal' )
        h.delete_points = 0;
    else
        h.delete_points = 1;
    end
    
    %Work out tolerance radius for selecting a point or creating a new
    %point
    r_tol = ( g.tmax - g.tmin )/100;
    
    %Get current point in graph coordinates (note t on graph is in ms)
    cp = get( h.AXESgraph,'currentpoint' );
    h.click_point_t = cp(1,1);
    h.click_point_s = cp(1,2);
    
    %Determine whether this point is within tolerance of an already
    %existing point, if so indicate which one this is. Otherwise create a
    %new point at the click point location
    dr = distance( [h.click_point_t ; h.click_point_s], [1000*d.t; d.s ] );
    
    %Either select, add or delete a point
    
    if ( h.delete_points == 1 ) && ( numel( d.t ) > 2 )
        
        %Mouse click is NOT single left, delete a point if within tolerance
        
        %Selected point is within tolerance
        if ~isempty ( find( dr < r_tol, 1 ) )
            %Determine index of closest point, this will now be selected
            [dr,indices] = sort(dr);
            h.selected_point_index = indices(1);
            
            %Delete this point
            d.t( h.selected_point_index ) = [];
            d.s( h.selected_point_index ) = [];
            
        end
        
        %No point is now selected
        h.selected_point_index = [];
    else
        
        %Select or add point (depending on tolerance of click to set of
        %points)
        
        %Selected point is outside tolerance
        if isempty ( find( dr < r_tol, 1 ) )
            %Create new point i.e. add to existing array
            d.t = [h.click_point_t/1000,d.t];
            d.s = [h.click_point_s,d.s];
            
            %This point is being selected
            h.selected_point_index = 1;
        else
            %Determine index of closest point, this will now be selected
            [dr,indices] = sort(dr);
            h.selected_point_index = indices(1);
        end
    end
end

%%

% Callback which runs when the mouse is moved in the figure window
function MouseDrag(hObject, eventdata, handles)
global d g h

%Check if in edit mode
if ( g.edit_mode == 1 ) && ( h.mouse_clicked == 1) && ( h.delete_points == 0 )
    
    %Get current point in graph coordinates
    cp = get( h.AXESgraph,'currentpoint' );
    h.click_point_t = cp(1,1);
    h.click_point_s = cp(1,2);
    
    %Move selected point to new location
    d.t(h.selected_point_index) = h.click_point_t/1000;
    d.s(h.selected_point_index) = h.click_point_s;
    gui_update;
end

%%

%%% CALLBACKS FOR GUI ELEMENTS %%%

% Waveform repetition frequency /Hz ( = reciprocal of the pulse duration )
% edit box
function EDITWRF_Callback(hObject, eventdata, handles)
global d h

%Modify period and stretch time data accordingly
new_WRF = str2num( get(h.EDITWRF,'string') );
new_period = 1/new_WRF;
d.t = (new_period/d.period)*d.t;
d.WRF = new_WRF;
d.period = new_period;
gui_update

%%

% Sample rate /kHz edit box
function EDITfs_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Maximum pulse amplitude value edit box
function EDITsmax_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Minimum pulse amplitude value edit box
function EDITsmin_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Minimum time /ms of pulse graph display
function EDITtmin_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Maximum time /ms of pulse graph display
function EDITtmax_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Tick mark interval /ms of pulse graph display
function EDITttick_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Tick mark interval of pulse aplitude display
function EDITstick_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Number of data points for f(t) generated waveform
function EDITN_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Hide points toggle
function CHECKhidepoints_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Restore deafult options pushbutton
function PUSHdefault_Callback(hObject, eventdata, handles)
global d g

%Get default parameters and update GUI
[g,d] = set_defaults;
gui_update

%%

% Toggle waveform editing with the mouse. Right mouse click will remove a
% nearby point (unless the number of points is three, which is the minimum)
% Left mouse click will either add a new point, or select a nearby point.
% Dragging the mouse while holding down the left button will move a point
% around. Multi-valued pulses are not allowed, so the amount of
% 'temporal'(time i.e. horizontal) movement allowed is set by the adjecent
% points.
function TOGGLEeditwaveform_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

% Edit box for frequency /Hz of waveform vs time /s
% Equation in edit box will involve parameters f, t, T
% T is the period in s = 1/WRF
function EDITfvst_Callback(hObject, eventdata, handles)
get_gui_data
gui_update

%%

%%% OTHER FUNCTIONS %%%

%distance
% Function which computes Euclidean distance matrix.
% This fully vectorized (VERY FAST!) m-file computes the
% Euclidean distance between two vectors by:
%          ||A-B|| = sqrt ( ||A||^2 + ||B||^2 - 2*A.B )
%
% Syntax: E = distance(A,B)
%    A - (DxM) matrix
%    B - (DxN) matrix
%    E - (MxN) Euclidean distances between vectors in A and B
%
% Example :
%    A = rand(400,100); B = rand(400,200);
%    d = distance(A,B);

% Author   : Roland Bunschoten
%            University of Amsterdam
%            Intelligent Autonomous Systems (IAS) group
%            Kruislaan 403  1098 SJ Amsterdam
%            tel.(+31)20-5257524
%            bunschot@wins.uva.nl
% Last Rev : Oct 29 16:35:48 MET DST 1999
% Tested   : PC Matlab v5.2 and Solaris Matlab v5.3
% Thanx    : Nikos Vlassis
function d = distance(a,b)
if (nargin ~= 2)
    error('Not enough input arguments');
end
if (size(a,1) ~= size(b,1))
    error('A and B should be of same dimensionality');
end
aa=sum(a.*a,1); bb=sum(b.*b,1); ab=a'*b;
d = sqrt(abs(repmat(aa',[1 size(bb,2)]) + repmat(bb,[size(aa,2) 1]) - 2*ab));

%%

%End of code